from annogesiclib.splice_parser import SpliceParser
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper


def get_feature(cds):
    '''get proper feature name'''
    if "locus_tag" in cds.attributes.keys():
        feature = cds.attributes["locus_tag"]
    elif "protein_id" in cds.attributes.keys():
        feature = cds.attributes["protein_id"]
    elif "ID" in cds.attributes.keys():
        strand = Helper().get_strand_name(cds.strand)
        feature = "".join([cds.attributes["ID"], ":",
                           str(cds.start), "-", str(cds.end),
                           "_", strand])
    else:
        strand = Helper().get_strand_name(cds.strand)
        feature = "".join([cds.feature, ":",
                           str(cds.start), "-", str(cds.end),
                           "_", strand])
    return feature


def detect_conflict(gffs, circ, num, out, out_best, args_circ):
    '''remove the false positive which overlap with known annotation'''
    detect = False
    gff = None
    for gff in gffs:
        if (gff.seq_id == circ.strain) and (
                gff.strand == circ.strand):
            if ((gff.start <= circ.start) and (
                 gff.end >= circ.start) and (
                 gff.end <= circ.end)) or (
                (gff.start >= circ.start) and (
                 gff.end <= circ.end)) or (
                (gff.start >= circ.start) and (
                 gff.start <= circ.end) and (
                 gff.end >= circ.end)) and (
                (gff.start <= circ.start) and (
                 gff.end >= circ.end)):
                detect = True
                break
    if detect:
        feature = get_feature(gff)
        out.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n".format(
                  "_".join(["circRNA", str(num)]), circ.strain, circ.strand,
                  circ.start, circ.end, feature, circ.supported_reads,
                  float(circ.supported_reads) / float(circ.start_site_reads),
                  float(circ.supported_reads) / float(circ.end_site_reads)))
    else:
        start_read = float(circ.supported_reads) / float(circ.start_site_reads)
        end_read = float(circ.supported_reads) / float(circ.end_site_reads)
        if (circ.supported_reads >= args_circ.support) and (
                start_read >= args_circ.start_ratio) and (
                end_read >= args_circ.end_ratio):
            out_best.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}"
                           "\t{6}\t{7}\t{8}\n".format(
                            "_".join(["circRNA", str(num)]), circ.strain,
                            circ.strand, circ.start, circ.end, "NA",
                            circ.supported_reads, start_read, end_read))
        out.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n".format(
                  "_".join(["circRNA", str(num)]), circ.strain, circ.strand,
                  circ.start, circ.end, "NA", circ.supported_reads,
                  start_read, end_read))
    return detect


def import_num(support, nums, strain):
    if support not in nums[strain].keys():
        nums[strain][support] = 0
    if support not in nums["all"].keys():
        nums["all"][support] = 0
    nums[strain][support] += 1
    nums["all"][support] += 1


def print_file(nums, stat, strain):
    for key in sorted(nums[strain].keys()):
        stat.write("\tthe number of potential circular RNAs, ")
        stat.write("more than {0} supported it = {1}\n".format(
                   key, nums[strain][key]))


def read_file(input_file, gff_file, hypo):
    circs = []
    gffs = []
    ps = SpliceParser()
    high = 0
    splice_fh = open(input_file)
    for entry in ps.parser(splice_fh):
        if entry.supported_reads > high:
            high = entry.supported_reads
        circs.append(entry)
    gff_parser = Gff3Parser()
    for entry in gff_parser.entries(open(gff_file)):
        if ("product" in entry.attributes.keys()) and (hypo):
            if "hypothetical protein" not in entry.attributes["product"]:
                gffs.append(entry)
        else:
            gffs.append(entry)
    gffs = sorted(gffs, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    circs = sorted(circs, key=lambda x: (x.strain, x.supported_reads),
                   reverse=True)
    splice_fh.close()
    return circs, gffs, high


def get_circrna(circs, gffs, high, out, out_best, args_circ):
    '''search the splice data to find the potential circRNA'''
    num_circular = {}
    num_circular["all"] = 0
    num_support = {}
    num_support["all"] = {}
    num_conflict = {}
    num_conflict["all"] = {}
    pre_seq_id = ""
    num = 0
    for circ in circs:
        if pre_seq_id != circ.strain:
            num_support[circ.strain] = {}
            num_conflict[circ.strain] = {}
            num_circular[circ.strain] = 0
        if (circ.situation != "F") and (circ.splice_type == "C"):
            num_circular[circ.strain] += 1
            num_circular["all"] += 1
            detect = detect_conflict(gffs, circ, num, out, out_best, args_circ)
            for support in range(0, high + 5, 5):
                if circ.supported_reads >= int(support):
                    import_num(support, num_support, circ.strain)
                    if detect is False:
                        if (float(circ.supported_reads) /
                            float(circ.start_site_reads) >=
                            args_circ.start_ratio) and (
                            float(circ.supported_reads) /
                            float(circ.end_site_reads) >=
                                args_circ.end_ratio):
                            import_num(support, num_conflict, circ.strain)
            num += 1
        pre_seq_id = circ.strain
    return {"circular": num_circular, "support": num_support,
            "conflict": num_conflict}


def detect_circrna(input_file, gff_file, output_file, args_circ, statistics):
    circs, gffs, high = read_file(input_file, gff_file, args_circ.hypo)
    out = open(output_file, "w")
    out.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n".format(
               "ID", "Genome", "Strand", "Start", "End", "Annotation_overlap",
               "Supported_reads", "Supported_reads/Reads_at_start",
               "Supported_reads/Reads_at_end"))
    out_best = open(output_file.replace("_all.csv", "_best.csv"), "w")
    out_best.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n".format(
               "ID", "Genome", "Strand", "Start", "End", "Annotation_overlap",
               "Supported_reads", "Supported_reads/Reads_at_start",
               "Supported_reads/Reads_at_end"))
    nums = get_circrna(circs, gffs, high, out, out_best, args_circ)
    stat = open(statistics, "w")
    stat.write("All genomes:\n")
    stat.write("\tBefore filtering:\n")
    stat.write("\tthe number of all circular RNAs = {0}\n".format(
               nums["circular"]["all"]))
    print_file(nums["support"], stat, "all")
    stat.write("\n\tAfter filtering:\n")
    stat.write("\t\twithout conflict with annotation\n")
    stat.write("\t\tsupport read ratio of starting "
               "point is larger than {0}\n".format(
                   args_circ.start_ratio))
    stat.write("\t\tsupport read ratio of end point "
               "is larger than {0}\n".format(
                   args_circ.end_ratio))
    print_file(nums["conflict"], stat, "all")
    if len(nums["circular"]) > 2:
        for strain in nums["circular"].keys():
            if strain != "all":
                stat.write("\n{0}:\n".format(strain))
                stat.write("\tBefore filtering:\n")
                stat.write("\tthe number of all circular RNAs = {0}\n".format(
                               nums["circular"][strain]))
                print_file(nums["support"], stat, strain)
                stat.write("\n\tAfter filtering:\n")
                stat.write("\t\twithout conflict with annotation\n")
                stat.write("\t\tsupport read ratio of starting point "
                           "is larger than {0}\n".format(
                               args_circ.start_ratio))
                stat.write("\t\tsupport read ratio of end point "
                           "is larger than {0}\n".format(
                               args_circ.end_ratio))
                print_file(nums["conflict"], stat, strain)
    out.close()
    out_best.close()
    stat.close()



from annogesiclib.helper import Helper
from annogesiclib.gff3 import Gff3Parser


def get_feature(gene, file_type):
    if "Name" in gene.attributes.keys():
        feature = gene.attributes["Name"]
    elif "locus_tag" in gene.attributes.keys():
        feature = gene.attributes["locus_tag"]
    else:
        if file_type != "tran":
            file_type = gene.feature
        strand = Helper().get_strand_name(gene.strand)
        feature = "".join([file_type, ":", str(gene.start),
                           "-", str(gene.end), "_", strand])
    pos = str(gene.start) + "-" + str(gene.end)
    return feature, pos


def import_data(f_1, f_2, start, end, file_type):
    if f_1 != "terminal":
        feature_1, pos_1 = get_feature(f_1, file_type)
        strain = f_1.seq_id
    else:
        feature_1 = "NA"
        pos_1 = "NA"
    if f_2 != "terminal":
        feature_2, pos_2 = get_feature(f_2, file_type)
        strain = f_2.seq_id
    else:
        feature_2 = "NA"
        pos_2 = "NA"
    return {"strain": strain, "start": start, "end": end,
            "parent_p": feature_1, "parent_m": feature_2, "print": False,
            "p_pos": pos_1, "m_pos": pos_2}


def get_terminal(genes, inters, gene_len, type_, file_type):
    '''deal with the intergenic region which located at two ends'''
    if type_ == "start":
        for gene in genes:
            if (gene.strand == "-"):
                inters.append(import_data(gene.strand, gene.seq_id, 1,
                                          gene.start, file_type))
                break
    elif type_ == "end":
        for gene in reversed(genes):
            if (gene.strand == "+"):
                inters.append(import_data(gene.strand, gene.seq_id, gene.end,
                                          gene_len, file_type))
                break


def get_inter(features, seq, file_type):
    '''compare features and get the intergenic region'''
    inters = []
    first = True
    pre_strain = ""
    pre_feature1 = None
    sort_features = sorted(features, key=lambda x: (x.seq_id, x.start,
                                                    x.end, x.strand))
    for feature1 in sort_features:
        if pre_strain != feature1.seq_id:
            if feature1.strand == "-":
                inters.append(import_data("terminal", feature1, 1,
                                          feature1.start, file_type))
            if not first:
                if pre_feature1.strand == "+":
                    inters.append(import_data(pre_feature1, "terminal",
                                  pre_feature1.start,
                                  len(seq[pre_feature1.seq_id]), file_type))
            if first:
                first = False
        for feature2 in sort_features:
            if (feature1.seq_id == feature2.seq_id) and (
                    feature1.end < feature2.end) and (
                    feature1.end >= feature2.start) and (
                    feature1.strand == feature2.strand):
                break
            elif (feature1.seq_id == feature2.seq_id) and (
                    feature1.end <= feature2.start) and (
                    feature1.strand == "+"):
                if feature1.strand == feature2.strand:
                    break
                else:
                    inters.append(import_data(feature1, feature2, feature1.end,
                                              feature2.start, file_type))
                    break
        pre_feature1 = feature1
        pre_strain = feature1.seq_id
    return inters


def import_merge(id_, strain, start, end, parent_p, parent_m, p_pos, m_pos):
    return {"ID": "_".join(["inter_" + str(id_)]), "strain": strain,
            "start": start, "end": end, "parent_p": parent_p,
            "parent_m": parent_m, "print": False, "p_pos": p_pos,
            "m_pos": m_pos}


def get_overlap_inters(inter1, inter2, merges, id_):
    if (inter1["end"] < inter2["end"]) and (
            inter1["end"] > inter2["start"]) and (
            inter1["start"] <= inter2["start"]):
        merges.append(import_merge(id_, inter1["strain"], inter2["start"],
                      inter1["end"], inter2["parent_p"], inter1["parent_m"],
                      inter2["p_pos"], inter1["m_pos"]))
        inter1["print"] = True
        inter2["print"] = True
        id_ += 1
    elif (inter1["start"] > inter2["start"]) and (
            inter1["start"] < inter2["end"]) and (
            inter1["end"] >= inter2["end"]):
        merges.append(import_merge(id_, inter1["strain"], inter1["start"],
                      inter2["end"], inter1["parent_p"], inter2["parent_m"],
                      inter1["p_pos"], inter2["m_pos"]))
        inter1["print"] = True
        inter2["print"] = True
        id_ += 1
    elif (inter1["end"] >= inter2["end"]) and (
            inter1["start"] <= inter2["start"]):
        merges.append(import_merge(id_, inter2["strain"], inter2["start"],
                      inter2["end"], inter2["parent_p"], inter2["parent_m"],
                      inter2["p_pos"], inter2["m_pos"]))
        inter1["print"] = True
        inter2["print"] = True
        id_ += 1
    elif (inter1["end"] <= inter2["end"]) and (
            inter1["start"] >= inter2["start"]):
        merges.append(import_merge(id_, inter1["strain"], inter1["start"],
                      inter1["end"], inter1["parent_p"], inter1["parent_m"],
                      inter1["p_pos"], inter1["m_pos"]))
        inter1["print"] = True
        inter2["print"] = True
        id_ += 1
    return id_


def merge_inter(inters1, inters2):
    '''check and merge the overlap intergenic seq'''
    merges = []
    id_ = 0
    for inter1 in inters1:
        for inter2 in inters2:
            if (inter1["strain"] == inter2["strain"]):
                id_ = get_overlap_inters(inter1, inter2, merges, id_)
        if not inter1["print"]:
            merges.append(import_merge(id_, inter1["strain"], inter1["start"],
                                       inter1["end"], inter1["parent_p"],
                                       inter1["parent_m"], inter1["p_pos"],
                                       inter1["m_pos"]))
            inter1["print"] = True
            id_ += 1
    for inter2 in inters2:
        if not inter2["print"]:
            merges.append(import_merge(id_, inter2["strain"], inter2["start"],
                                       inter2["end"], inter2["parent_p"],
                                       inter2["parent_m"], inter2["p_pos"],
                                       inter2["m_pos"]))
            inter2["print"] = True
            id_ += 1
    sort_merges = sorted(merges, key=lambda x: (x["strain"],
                                                x["start"], x["end"]))
    return sort_merges


def detect_confliction(gc, genes, seq):
    '''check the intergenic region 
    (add the region for folding proper sec str.) 
    will overlap with gene or not'''
    corr_merges = []
    overlap = False
    tmp_start = gc["start"]
    tmp_end = gc["end"]
    if "tran" in gc["parent_p"]:
        if gc["start"] > 80:
            for gene in genes:
                if ((gc["start"] - 80) > gene.start) and (
                        (gc["start"] - 80) < gene.end) and (
                        gene.strand == "+"):
                    tmp_start = gene.end - 30
                    overlap = True
            if not overlap:
                tmp_start = gc["start"] - 80
        else:
            tmp_start = 1
    else:
        if gc["start"] > 30:
            tmp_start = gc["start"] - 30
        else:
            tmp_start = 1
    corr_merges.append(import_merge(gc["ID"], gc["strain"], tmp_start,
                       gc["end"], gc["parent_p"], gc["parent_m"],
                       gc["p_pos"], gc["m_pos"]))
    corr_merges[-1]["strand"] = "+"
    if "tran" in gc["parent_m"]:
        if gc["end"] < (len(seq[gc["strain"]]) - 80):
            for gene in genes:
                if ((gc["end"] + 80) > gene.start) and (
                        (gc["end"] + 80) < gene.end) and (
                        gene.strand == "-"):
                    tmp_end = gene.start + 30
                    overlap = True
            if not overlap:
                tmp_end = gc["end"] + 80
        else:
            tmp_end = len(seq[gc["strain"]])
    else:
        if gc["end"] < len(seq[gc["strain"]]) - 30:
            tmp_end = gc["end"] + 30
        else:
            tmp_end = len(seq[gc["strain"]])
    corr_merges.append(import_merge(gc["ID"], gc["strain"], gc["start"],
                       tmp_end, gc["parent_p"], gc["parent_m"],
                       gc["p_pos"], gc["m_pos"]))
    corr_merges[-1]["strand"] = "-"
    return corr_merges


def read_file(seq_file, tran_file, gff_file):
    seq = {}
    tas = []
    genes = []
    merges = []
    with open(seq_file, "r") as f_h:
        for line in f_h:
            line = line.strip()
            if line.startswith(">"):
                strain = line[1:]
                seq[strain] = ""
            else:
                seq[strain] = seq[strain] + line
    ta_fh = open(tran_file, "r")
    for entry in Gff3Parser().entries(ta_fh):
        tas.append(entry)
        merges.append(entry)
    for entry in Gff3Parser().entries(open(gff_file)):
        if (entry.feature == "gene"):
            genes.append(entry)
            merges.append(entry)
    tas = sorted(tas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    genes = sorted(genes, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return seq, tas, merges, genes


def get_fasta(seq, merge, num, strand, args_term, out, out_i):
    if (merge["end"] - merge["start"]) > args_term.window:
        detect_out = False
        for start in range(merge["start"], merge["end"] + 1, args_term.shift):
            if (merge["end"] - (start + args_term.window)) < args_term.shift:
                end = merge["end"]
                detect_out = True
            else:
                end = start + args_term.window
            inter_seq = Helper().extract_gene(
                seq[merge["strain"]], start, end, strand)
            out_i.write(">" + "|".join([
                "inter_" + str(num), str(start),
                str(end), merge["strain"], merge["parent_p"],
                merge["parent_m"], merge["p_pos"], merge["m_pos"],
                strand]) + "\n")
            out.write(">inter_" + str(num) + "\n")
            out.write(inter_seq + "\n")
            num += 1
            if detect_out:
                break
    else:
        inter_seq = Helper().extract_gene(
            seq[merge["strain"]], merge["start"], merge["end"], strand)
        out_i.write(">" + "|".join([
            "inter_" + str(num), str(merge["start"]),
            str(merge["end"]), merge["strain"], merge["parent_p"],
            merge["parent_m"], merge["p_pos"], merge["m_pos"],
            strand]) + "\n")
        out.write(">inter_" + str(num) + "\n")
        out.write(inter_seq + "\n")
        num += 1
    return num


def mod_inter_tas_gene(inter_tas, genes):
    for inter in inter_tas:
        for gene in genes:
            if (inter["strain"] == gene.seq_id):
                if (gene.end > inter["start"]) and (
                        gene.end < inter["end"]) and (
                        gene.strand == "+"):
                    inter["start"] = gene.end
                    inter["parent_p"], inter["p_pos"] = get_feature(
                            gene, "gene")
                elif (gene.start > inter["start"]) and (
                        gene.start < inter["end"]) and (
                        gene.strand == "-"):
                    inter["end"] = gene.start
                    inter["parent_m"], inter["m_pos"] = get_feature(
                            gene, "gene")
                    break


def intergenic_seq(seq_file, tran_file, gff_file, out_file,
                   index_file, args_term):
    '''get intergenic seq'''
    out = open(out_file, "w")
    out_i = open(index_file, "w")
    seq, tas, merges, genes = read_file(seq_file, tran_file, gff_file)
    inter_tas = get_inter(tas, seq, "tran")
    mod_inter_tas_gene(inter_tas, genes)
    inter_genes = get_inter(genes, seq, "gene")
    merges = merge_inter(inter_tas, inter_genes)
    num = 0
    for tmp_merge in merges:
        corr_merges = detect_confliction(tmp_merge, genes, seq)
        for merge in corr_merges:
            if merge["start"] < merge["end"]:
                if merge["strand"] == "+":
                    num = get_fasta(seq, merge, num, "+", args_term,
                                    out, out_i)
                else:
                    num = get_fasta(seq, merge, num, "-", args_term,
                                    out, out_i)



import csv


def read_file(srna_file, nums):
    srna_f = open(srna_file, "r")
    for row in csv.reader(srna_f, delimiter="\t"):
        if (row[-6] != "NA") and (row[0] != "Rank"):
            if row[1] not in nums.keys():
                nums[row[1]] = {}
            if row[2] not in nums[row[1]].keys():
                nums[row[1]][row[2]] = 1
            else:
                nums[row[1]][row[2]] += 1
            if row[2] not in nums["total"].keys():
                nums["total"][row[2]] = 1
            else:
                nums["total"][row[2]] += 1
    srna_f.close()


def blast_class(srna_file, out_file):
    '''statistics of the results of blast sRNA database'''
    nums = {}
    nums["total"] = {}
    read_file(srna_file, nums)
    out = open(out_file, "w")
    if len(nums) > 1:
        if len(nums) > 2:
            out.write("All genomes:\n")
            out.write("sRNA_name\tamount\n")
            for blast, num in nums["total"].items():
                out.write("{0}\t{1}\n".format(blast, num))
        for strain, srna_name in nums.items():
            if strain != "total":
                out.write(strain + ":\n")
                out.write("sRNA_name\tamount\n")
                for blast, num in srna_name.items():
                    out.write("{0}\t{1}\n".format(blast, num))
    else:
        out.write("No known sRNA!!\n")
    out.close()



import copy


def coverage_comparison(cover, cover_sets, poss, first, strand, cover_pos):
    '''Seaching the lowest and highest coverage'''
    if first:
        first = False
        cover_sets["high"] = cover
        cover_sets["low"] = cover
        poss["high"] = cover_pos
        poss["low"] = cover_pos
    else:
        if cover_sets["high"] < cover:
            cover_sets["high"] = cover
            poss["high"] = cover_pos
            poss["low"] = cover_pos
            cover_sets["low"] = cover
        if ((strand == "+") and (poss["low"] >= poss["high"])) or \
           ((strand == "-") and (poss["low"] <= poss["high"])):
            if cover_sets["low"] > cover:
                cover_sets["low"] = cover
                poss["low"] = cover_pos
        elif ((strand == "+") and (poss["low"] < poss["high"])) or \
             ((strand == "-") and (poss["low"] > poss["high"])):
            poss["low"] = cover_pos
            cover_sets["low"] = cover
    return first


def get_repmatch(replicates, cond):
    '''deal with the replicate match'''
    detect_all = False
    for rep in replicates:
        if "all" in rep:
            detect_all = True
            rep = int(rep.split("_")[-1])
            break
    if not detect_all:
        for match in replicates:
            if cond.split("_")[0] == match.split("_")[0]:
                rep = int(match.split("_")[-1])
    return rep


def define_cutoff(coverages, median, utr_type):
    '''get the cutoff'''
    cutoffs = {}
    if coverages[utr_type] == "mean":
        for track, values in median.items():
            cutoffs[track] = values["mean"]
    else:
        for track, values in median.items():
            cutoffs[track] = values["median"]
    return cutoffs


def check_notex(cover, texs, cutoff, notex):
    '''Check the cutoff of average coverage for TEX+ and TEX- libs'''
    if notex is not None:
        for keys in texs.keys():
            tracks = keys.split("@AND@")
            if cover["track"] == tracks[0]:
                if cover["avg"] > cutoff:
                    return True
            elif cover["track"] == tracks[1]:
                if cover["avg"] > notex:
                    return True
    else:
        if cover["avg"] > cutoff:
            return True


def run_tex(cover, texs, check_texs, tex_notex, type_,
            detect_num, poss, target_datas):
    '''Check the position of different libs'''
    if (cover["type"] == "tex") or (cover["type"] == "notex"):
        for key in texs.keys():
            if cover["track"] in key:
                texs[key] += 1
                check_texs[key].append(cover)
                if texs[key] >= tex_notex:
                    if type_ == "sRNA_utr_derived":
                        if detect_num == 0:
                            poss["start"] = cover["final_start"]
                            poss["end"] = cover["final_end"]
                        else:
                            exchange_start_end(poss, cover)
                    detect_num += 1
                    if cover not in target_datas:
                        target_datas.append(cover)
                    if tex_notex != 1:
                        if check_texs[key][0] not in target_datas:
                            target_datas.append(check_texs[key][0])
                            if type_ == "sRNA_utr_derived":
                                exchange_start_end(poss, check_texs[key][0])
    elif cover["type"] == "frag":
        if type_ == "sRNA_utr_derived":
            if detect_num == 0:
                poss["start"] = cover["final_start"]
                poss["end"] = cover["final_end"]
            else:
                exchange_start_end(poss, cover)
        detect_num += 1
        target_datas.append(cover)
    return detect_num


def check_tex(template_texs, covers, target_datas, notex, type_, poss, median,
              coverages, utr_type, cutoff_coverage, tex_notex):
    '''Check the candidates for TEX+/- libs 
    (should be detected in one or both of libs)'''
    detect_num = 0
    check_texs = {}
    texs = copy.deepcopy(template_texs)
    for key, num in texs.items():
        check_texs[key] = []
    for cover in covers:
        run_check_tex = False
        if type_ == "sRNA_utr_derived":
            cutoffs = define_cutoff(coverages, median, utr_type)
            if cover["track"] in cutoffs.keys():
                if cover["avg"] > cutoffs[cover["track"]]:
                    run_check_tex = True
            else:
                run_check_tex = True
        elif type_ == "sORF":
            if cover["avg"] > coverages[cover["track"]]:
                run_check_tex = True
        elif (type_ == "terminator"):
            run_check_tex = True
        elif (type_ == "normal"):
            run_check_tex = check_notex(cover, texs, cutoff_coverage,
                                        notex)
        else:
            if cover["avg"] > cutoff_coverage:
                run_check_tex = True
        if run_check_tex:
            detect_num = run_tex(cover, texs, check_texs, tex_notex,
                                 type_, detect_num, poss, target_datas)
    return detect_num


def exchange_start_end(poss, cover):
    '''modify the start and end point. get the long one'''
    if poss["start"] > cover["final_start"]:
        poss["start"] = cover["final_start"]
    if poss["end"] < cover["final_end"]:
        poss["end"] = cover["final_end"]


def replicate_comparison(args_srna, srna_covers, strand, type_, median,
                         coverages, utr_type, notex, cutoff_coverage, texs):
    '''Check the number of replicates which fit the cutoff in order to remove 
    the candidates which only can be detected in few replicates.'''
    srna_datas = {"best": 0, "high": 0, "low": 0, "start": -1,
                  "end": -1, "track": "", "detail": [], "conds": {}}
    tmp_poss = {"start": -1, "end": -1, "pos": -1,
                "all_start": [], "all_end": []}
    detect = False
    for cond, covers in srna_covers.items():
        detect_num = check_tex(
                texs, covers, srna_datas["detail"], notex, type_, tmp_poss,
                median, coverages, utr_type, cutoff_coverage,
                args_srna.tex_notex)
        if ("texnotex" in cond):
            tex_rep = get_repmatch(args_srna.replicates["tex"], cond)
            if detect_num >= tex_rep:
                detect = True
        elif ("frag" in cond):
            frag_rep = get_repmatch(args_srna.replicates["frag"], cond)
            if detect_num >= frag_rep:
                detect = True
        if detect:
            detect = False
            if type_ == "sRNA_utr_derived":
                tmp_poss["all_start"].append(tmp_poss["start"])
                tmp_poss["all_end"].append(tmp_poss["end"])
            else:
                if strand == "+":
                    sort_datas = sorted(srna_datas["detail"],
                                        key=lambda k: (k["pos"]))
                else:
                    sort_datas = sorted(srna_datas["detail"],
                                        key=lambda k: (k["pos"]), reverse=True)
                srna_datas["pos"] = sort_datas[-1]["pos"]
            sort_datas = sorted(srna_datas["detail"], key=lambda k: (k["avg"]))
            avg = sort_datas[-1]["avg"]
            srna_datas["conds"][cond] = str(detect_num)
            if (avg > srna_datas["best"]):
                srna_datas["high"] = sort_datas[-1]["high"]
                srna_datas["low"] = sort_datas[-1]["low"]
                srna_datas["best"] = avg
                srna_datas["track"] = sort_datas[-1]["track"]
    if type_ == "sRNA_utr_derived":
        if len(tmp_poss["all_start"]) != 0:
            srna_datas["start"] = min(tmp_poss["all_start"])
            srna_datas["end"] = max(tmp_poss["all_end"])
        else:
            srna_datas["start"] = -1
            srna_datas["end"] = -1
    return srna_datas



import os


def print_track(track_num, svg_out, figure_width):
    id_num = 3067
    x = 2.0744663
    y = 131
    for track in range(track_num):
        if (track % 2) == 0:
            svg_out.write("  <rect\n")
            svg_out.write("     width=\"{0}\"\n".format(figure_width))
            svg_out.write("     height=\"40\"\n")
            svg_out.write("     x=\"{0}\"\n".format(x))
            if track == 0:
                svg_out.write("     y=\"{0}\"\n".format(y))
            else:
                y = y + 40
                svg_out.write("     y=\"{0}\"\n".format(y))
            svg_out.write("     id=\"rect{0}\"\n".format(id_num))
            svg_out.write("     style=\"opacity:0.25;fill:#37c84f;"
                          "fill-opacity:0.25;fill-rule:evenodd;")
            svg_out.write("stroke:#000000;stroke-width:1px;"
                          "stroke-linecap:butt;stroke-linejoin:miter;"
                          "stroke-opacity:0.25\" />\n")
        if (track % 2) == 1:
            svg_out.write("  <rect\n")
            svg_out.write("     width=\"{0}\"\n".format(figure_width))
            svg_out.write("     height=\"40\"\n")
            svg_out.write("     x=\"{0}\"\n".format(x))
            y = y + 40
            svg_out.write("     y=\"{0}\"\n".format(y))
            svg_out.write("     id=\"rect{0}\"\n".format(id_num))
            svg_out.write("     style=\"opacity:0.25;fill:#c8374f;"
                          "fill-opacity:0.25;fill-rule:evenodd;")
            svg_out.write("stroke:#000000;stroke-width:1px;"
                          "stroke-linecap:butt;stroke-linejoin:miter;"
                          "stroke-opacity:0.25\" />\n")
        id_num += 1


def gen_svg(input_png, track_num, figure_height, figure_width):
    svg_out = open(input_png[:-4] + ".svg", "w")
    svg_out.write("""<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!-- Created with Inkscape (http://www.inkscape.org/) -->

<svg
   xmlns:dc="http://purl.org/dc/elements/1.1/"
   xmlns:cc="http://creativecommons.org/ns#"
   xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
   xmlns:svg="http://www.w3.org/2000/svg"
   xmlns="http://www.w3.org/2000/svg"
   xmlns:xlink="http://www.w3.org/1999/xlink"
   version="1.1"
""")
    svg_out.write("   width=\"{0}\"\n".format(figure_width))
    svg_out.write("   height=\"{0}\"\n".format(figure_height))
    svg_out.write("   viewBox=\"0 0 1860 {0}\"\n".format(figure_height))
    svg_out.write("""   id="svg3055">
  <metadata
     id="metadata3061">
    <rdf:RDF>
      <cc:Work
         rdf:about="">
        <dc:format>image/svg+xml</dc:format>
        <dc:type
           rdf:resource="http://purl.org/dc/dcmitype/StillImage" />
        <dc:title></dc:title>
      </cc:Work>
    </rdf:RDF>
  </metadata>
  <defs
     id="defs3059" />
  <image
""")
    svg_out.write("     xlink:href=\"file://{0}/{1}\"\n".format(
                  os.getcwd(), input_png))
    svg_out.write("""     width="100%"
     height="100%"
     preserveAspectRatio="xMidYMin meet"
     id="image3063" />\n""")
    print_track(track_num, svg_out, figure_width)
    svg_out.write("</svg>")
    svg_out.close()



import os
import itertools
from annogesiclib.gff3 import Gff3Parser


def print_intersection(datas, keys, num_srna, gff_name, type_, out_stat):
    num = 0
    datas_merge = []
    if type_ == "total":
        out = open(gff_name, "w")
        out.write("##gff-version 3\n")
    for data in datas[keys[0]]:
        check_same = []
        for key in keys[1:]:
            if data in datas[key]:
                check_same.append("True")
        if len(check_same) == (len(keys) - 1):
            if len(keys) <= 5:
                if type_ == "total":
                    out.write(data.info + "\t" + "\n")
            if "best_avg_coverage" in data.attributes.keys():
                datas_merge.append({
                    "data": data,
                    "wig": data.attributes["best_avg_coverage"]})
            num += 1
    datas_sort = sorted(datas_merge, key=lambda k: float(k['wig']),
                        reverse=True)
    for data in datas_sort:
        if type_ == "total":
            out.write(data["data"].info + "\n")
    if num_srna == 0:
        out_stat.write("\t{0} = {1}({2})\n".format(" and ".join(keys),
                       str(num), str(0)))
    else:
        out_stat.write("\t{0} = {1}({2})\n".format(" and ".join(keys),
                       str(num), str(float(num)/float(num_srna))))
    if type_ == "total":
        out.close()


def initiate(key, key_list, class_name, class_num, index, out, content):
    if key in key_list:
        class_num += 1
        index[class_name] = class_num
        out.write(str(class_num) + content + "\n")
    return class_num


def create_class(data, energy, datas_srna, index, type_, nr_hits_num):
    if "2d_energy" in data.attributes.keys():
        if float(data.attributes["2d_energy"]) < energy:
            datas_srna["class_" + str(index["2d_energy"])].append(data)
    if "with_TSS" in data.attributes.keys():
        if data.attributes["with_TSS"] != "NA":
            datas_srna["class_" + str(index["with_TSS"])].append(data)
        elif ((type_ == "UTR_derived") or (type_ == "total")) and (
                ("5utr" in data.attributes["sRNA_type"]) or (
                 "3utr" in data.attributes["sRNA_type"]) or (
                 "interCDS" in data.attributes["sRNA_type"])):
            if (data.attributes["start_cleavage"] != "NA") and (
                    ("3utr" in data.attributes["sRNA_type"]) or (
                     "interCDS" in data.attributes["sRNA_type"])):
                datas_srna["class_" + str(index["with_TSS"])].append(data)
        if "nr_hit" in data.attributes.keys():
            if ((data.attributes["nr_hit"] != "NA") and (
                    int(data.attributes["nr_hit"]) <= nr_hits_num)) or (
                    data.attributes["nr_hit"] == "NA"):
                datas_srna["class_" + str(index["nr_no_hit"])].append(data)
        if "sORF" in data.attributes.keys():
            if (data.attributes["sORF"] == "NA"):
                datas_srna["class_" + str(index["sORF"])].append(data)
        if "sRNA_hit" in data.attributes.keys():
            if data.attributes["sRNA_hit"] != "NA":
                datas_srna["class_" + str(index["sRNA_hit"])].append(data)
            else:
                datas_srna["class_" + str(index["sRNA_no_hit"])].append(data)
        if "with_term" in data.attributes.keys():
            if (data.attributes["with_term"] != "NA"):
                datas_srna["class_" + str(index["with_term"])].append(data)
            elif ("end_cleavage" in data.attributes.keys()):
                if data.attributes["end_cleavage"] != "NA":
                    datas_srna["class_" + str(index["with_term"])].append(data)
        if "promoter" in data.attributes.keys():
            if (data.attributes["promoter"] != "NA"):
                datas_srna["class_" + str(index["promoter"])].append(data)


def import_class(class_num, datas_srna, datas, index, num_srna, strain,
                 type_, srna_type, energy, nr_hits_num):
    for num in range(1, class_num + 1):
        datas_srna["class_" + str(num)] = []
    for data in datas[strain]:
        detect = False
        if (srna_type in data.attributes["sRNA_type"]) or (type_ == "total"):
            if type_ == "UTR_derived":
                if srna_type in data.attributes["sRNA_type"]:
                    detect = True
            else:
                detect = True
            if detect:
                num_srna += 1
                create_class(data, energy, datas_srna, index,
                             type_, nr_hits_num)
    return num_srna


def import_data(class_num, datas, index, num_srna,
                strain, checks, energy, nr_hits_num):
    datas_srna = {}
    if checks["utr"]:
        datas_srna["5'UTR_derived"] = {}
        num_srna["5'UTR_derived"] = import_class(
                class_num, datas_srna["5'UTR_derived"], datas, index,
                num_srna["5'UTR_derived"], strain, "UTR_derived",
                "5utr", energy, nr_hits_num)
        datas_srna["3'UTR_derived"] = {}
        num_srna["3'UTR_derived"] = import_class(
                class_num, datas_srna["3'UTR_derived"], datas, index,
                num_srna["3'UTR_derived"], strain, "UTR_derived",
                "3utr", energy, nr_hits_num)
        datas_srna["interCDS"] = {}
        num_srna["interCDS"] = import_class(
                class_num, datas_srna["interCDS"], datas, index,
                num_srna["interCDS"], strain, "UTR_derived",
                "interCDS", energy, nr_hits_num)
    if checks["inter"]:
        datas_srna["intergenic"] = {}
        num_srna["intergenic"] = import_class(
                class_num, datas_srna["intergenic"], datas, index,
                num_srna["intergenic"], strain, "intergenic",
                "intergenic", energy, nr_hits_num)
    if checks["in_CDS"]:
        datas_srna["in_CDS"] = {}
        num_srna["in_CDS"] = import_class(
                class_num, datas_srna["in_CDS"], datas, index,
                num_srna["in_CDS"], strain, "in_CDS",
                "in_CDS", energy, nr_hits_num)
    if checks["antisense"]:
        datas_srna["antisense"] = {}
        num_srna["antisense"] = import_class(
                class_num, datas_srna["antisense"], datas, index,
                num_srna["antisense"], strain, "antisense",
                "antisense", energy, nr_hits_num)
    datas_srna["total"] = {}
    num_srna["total"] = import_class(
            class_num, datas_srna["total"], datas, index,
            num_srna["total"], strain, "total", "total",
            energy, nr_hits_num)
    return datas_srna


def sort_keys(keys):
    nums = []
    final_keys = []
    for key in keys:
        nums.append(int(key.split("_")[1]))
    nums = sorted(nums)
    for num in nums:
        final_keys.append("_".join(["class", str(num)]))
    return final_keys


def print_stat_title(checks, out_stat, strain, srna_datas,
                     num_strain, args_srna):
    class_num = 0
    index = {}
    if checks["first"]:
        checks["first"] = False
        class_num = initiate(
                "2d_energy", srna_datas[strain][0].attributes.keys(),
                "2d_energy", class_num, index, out_stat,
                " - the normalized(by length of sRNA) free energy "
                "change of secondary structure below to " +
                str(args_srna.energy))
        name = " ".join([
            " - sRNA candidates start with TSS",
            "(3'UTR derived and interCDS sRNA also includes the sRNA "
            "candidates which start with processing site.)"])
        class_num = initiate("tss", args_srna.import_info, "with_TSS",
                             class_num, index, out_stat, name)
        class_num = initiate(
                "nr_hit", srna_datas[strain][0].attributes.keys(),
                "nr_no_hit", class_num, index, out_stat,
                "".join([" - blast can not find the homology from nr "
                         "database (the cutoff is ",
                         str(args_srna.nr_hits_num), ")."]))
        class_num = initiate(
                "with_term", srna_datas[strain][0].attributes.keys(),
                "with_term", class_num, index, out_stat,
                " - sRNA candidates ends with terminator (including the "
                "candidates ends with processing site).")
        class_num = initiate(
                "sORF", srna_datas[strain][0].attributes.keys(),
                "sORF", class_num, index, out_stat,
                " - have no confliction of sORF candidates.")
        class_num = initiate(
                "sRNA_hit", srna_datas[strain][0].attributes.keys(),
                "sRNA_no_hit", class_num, index, out_stat,
                " - blast can not find the homology from sRNA database.")
        class_num = initiate(
                "sRNA_hit", srna_datas[strain][0].attributes.keys(),
                "sRNA_hit", class_num, index, out_stat,
                " - blast can find the homology from sRNA database.")
        class_num = initiate(
                "promoter", srna_datas[strain][0].attributes.keys(),
                "promoter", class_num, index, out_stat,
                " - sRNA candidates associated with promoter.")
    else:
        out_stat.write("\n")
    if num_strain <= 2:
        out_stat.write("All genomes:\n")
        checks["limit"] = True
    else:
        if strain == "all":
            out_stat.write("All genomes:\n")
        else:
            out_stat.write(strain + ":\n")
    return class_num, index


def read_file(srna_file):
    strains = []
    checks = {"limit": False, "first": True, "utr": False,
              "inter": False, "in_CDS": False, "antisense": False}
    srna_datas = {}
    srna_datas["all"] = []
    strains.append("all")
    pre_seq_id = ""
    fh = open(srna_file)
    for entry in Gff3Parser().entries(fh):
        if ("5utr" in entry.attributes["sRNA_type"]) or (
                "3utr" in entry.attributes["sRNA_type"]) or (
                "interCDS" in entry.attributes["sRNA_type"]):
            checks["utr"] = True
        elif "intergenic" in entry.attributes["sRNA_type"]:
            checks["inter"] = True
        elif entry.attributes["sRNA_type"] == "in_CDS":
            checks["in_CDS"] = True
        elif "antisense" in entry.attributes["sRNA_type"]:
            checks["antisense"] = True
        if entry.seq_id != pre_seq_id:
            srna_datas[entry.seq_id] = []
            strains.append(entry.seq_id)
            pre_seq_id = entry.seq_id
        srna_datas[entry.seq_id].append(entry)
        srna_datas["all"].append(entry)
    for strain in srna_datas.keys():
        srna_datas[strain] = sorted(
                srna_datas[strain],
                key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    fh.close()
    return srna_datas, strains, checks


def set_num(num_srna, types):
    for type_ in types:
        num_srna[type_] = 0


def check_and_set_num(checks):
    num_srna = {"total": 0}
    if checks["utr"]:
        set_num(num_srna, ["5'UTR_derived", "3'UTR_derived", "interCDS"])
    if checks["antisense"]:
        set_num(num_srna, ["antisense"])
    if checks["inter"]:
        set_num(num_srna, ["intergenic"])
    if checks["in_CDS"]:
        set_num(num_srna, ["in_CDS"])
    return num_srna


def classify_srna(srna_file, out_folder, out_stat_file, args_srna):
    '''classify the sRNA based on the filters'''
    srna_datas, strains, checks = read_file(srna_file)
    out_stat = open(out_stat_file, "w")
    for strain in strains:
        checks["first"] = True
        if checks["limit"] is True:
            break
        class_num = 0
        num_srna = check_and_set_num(checks)
        if args_srna.in_cds:
            num_srna["in_CDS"] = 0
        class_num, index = print_stat_title(
                checks, out_stat, strain, srna_datas, len(strains), args_srna)
        srna_class = import_data(
                class_num, srna_datas, index, num_srna, strain,
                checks, args_srna.energy, args_srna.nr_hits_num)
        for type_, srna in num_srna.items():
            out_stat.write("sRNA type - {0}:\n".format(type_))
            out_stat.write("\ttotal sRNA candidates = {0}\n".format(srna))
            for num in range(1, class_num + 1):
                if srna != 0:
                    out_stat.write("\tclass {0} = {1}({2})\n".format(
                        num, len(srna_class[type_]["class_" + str(num)]),
                        float(len(srna_class[type_]["class_" + str(num)])) /
                        float(srna)))
                elif srna == 0:
                    out_stat.write("\tclass {0} = {1}({2})\n".format(
                        num, len(srna_class[type_]["class_" + str(num)]), 0))
                if type_ == "total":
                    out = open(os.path.join(
                        out_folder, "_".join(["class", str(num),
                                              strain + ".gff"])), "w")
                    out.write("##gff-version 3\n")
                    for data in (
                            srna_class[type_]["_".join(["class", str(num)])]):
                        out.write(data.info + "\n")
            if class_num >= 2:
                for comb in range(2, class_num):
                    for keys in itertools.combinations(
                            srna_class[type_].keys(), comb):
                        if (("class_" + str(index["sRNA_hit"])) in keys) and (
                               ("class_" + str(index["sRNA_no_hit"])) in keys):
                            continue
                        else:
                            keys = sort_keys(list(keys))
                            gff_name = os.path.join(
                                    out_folder, "_".join(sorted(list(keys)) +
                                                         [strain]) + ".gff")
                            print_intersection(
                                srna_class[type_], keys, srna,
                                gff_name, type_, out_stat)
    out_stat.close()
    out.close()



import csv


class Gff3Parser(object):
    """
A format description can be found at:
http://genome.ucsc.edu/FAQ/FAQformat.html#format3
http://www.sequenceontology.org/gff3.shtml

a validator can be found here:
http://modencode.oicr.on.ca/cgi-bin/validate_gff3_online

WARNING: Currently this class in not strict enough and would also
parse file not following the standard.
"""

    def entries(self, input_gff_fh):
        """
"""
        for entry_dict in csv.DictReader(
            input_gff_fh, delimiter="\t",
            fieldnames=["seq_id", "source", "feature", "start",
                        "end", "score", "strand", "phase", "attributes"]):
            if entry_dict["seq_id"].startswith("#"):
                continue
            yield self._dict_to_entry(entry_dict)

    def _dict_to_entry(self, entry_dict):
        return Gff3Entry(entry_dict)


class Gff3Entry(object):

    """

Example:
start, end = sorted([int(pos) for pos in [start, end]])
Gff3Entry({
"seq_id" : seq_id,
"source" : "MyLab",
"feature" : "sRNA",
"start" : start,
"end" : end,
"strand" : strand,
"score" : ".",
"phase" : ".",
"attributes" : "name=%s;locus_tag=%s" % (name, locus_tag)})
"""

    def __init__(self, entry_dict):
        self.seq_id = entry_dict["seq_id"]
        self.source = entry_dict["source"]
        self.feature = entry_dict["feature"]
        # 1-based coordinates
        # Make sure that start <= end
        start, end = sorted([int(entry_dict["start"]), int(entry_dict["end"])])
        self.start = start
        self.end = end
        self.score = entry_dict["score"]
        self.strand = entry_dict["strand"]
        self.phase = entry_dict["phase"]
        self.attributes = self._attributes(entry_dict["attributes"])
        self.attribute_string = entry_dict["attributes"]
        self.info = "\t".join([str(field) for field in [
                        self.seq_id, self.source, self.feature, self.start,
                        self.end, self.score, self.strand, self.phase,
                        self.attribute_string]])
        self.info_without_attributes = "\t".join([str(field) for field in [
                        self.seq_id, self.source, self.feature, self.start,
                        self.end, self.score, self.strand, self.phase]])

    def _attributes(self, attributes_string):
        """Translate the attribute string to dictionary"""
        attributes = {}
        if len(attributes_string) > 0:
            for attribute in attributes_string.split(";"):
                key_value_pair = attribute.split("=")
                key = key_value_pair[0]
                if len(key_value_pair) > 2:
                    value = "=".join(key_value_pair[1:])
                else:
                    value = key_value_pair[1]
                attributes[key] = value
            return attributes
        else:
            return attributes

    def add_attribute(self, key, value):
        self.attributes[key] = value
        self.attribute_string = ";".join(
            ["=".join(items) for items in self.attributes.items()])

    def __str__(self):
        return "\t".join([str(field) for field in [
                        self.seq_id, self.source, self.feature, self.start,
                        self.end, self.score, self.strand, self.phase,
                        self.attribute_string]])



import os
import shutil
import sys
import csv
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
from Bio.Seq import Seq
from annogesiclib.seqmodifier import SeqModifier


class SeqEditer(object):
    '''Edit the sequence if it is needed'''

    def _row_to_location(self, row):
        return({"target_id": row[1], "ref_id": row[0],
                "datas": [{"ref_nt": row[2],
                           "tar_nt": row[4], "position": row[3]}]})

    def _import_data(self, mod_table_file):
        datas = []
        first = True
        num_index = 0
        fh = open(mod_table_file)
        for row in csv.reader(fh, delimiter="\t"):
            if row[0].startswith("#"):
                continue
            else:
                if first:
                    datas.append(self._row_to_location(row))
                    pre_ref_id = row[0].strip()
                    pre_tar_id = row[1].strip()
                    first = False
                else:
                    if (row[0] == pre_ref_id) and \
                       (row[1] == pre_tar_id):
                        datas[num_index]["datas"].append(
                              {"ref_nt": row[2].strip(),
                               "tar_nt": row[4].strip(),
                               "position": row[3].strip()})
                    else:
                        datas.append(self._row_to_location(row))
                        num_index += 1
                        pre_ref_id = row[0].strip()
                        pre_tar_id = row[1].strip()
        fh.close()
        return datas

    def modify_seq(self, fasta_folder, mod_table_file, output_folder):
        datas = self._import_data(mod_table_file)
        for data in datas:
            seq = ""
            if (data["ref_id"] + ".fa") in os.listdir(fasta_folder):
                filename = os.path.join(fasta_folder, data["ref_id"] + ".fa")
                with open(filename, "r") as fasta:
                    for line in fasta:
                        line = line.strip()
                        if line[0] != ">":
                            seq = seq + line
                seq_modifier = SeqModifier(seq)
                for change in data["datas"]:
                    if change["ref_nt"] == "-":
                        seq_modifier.insert(
                                     int(change["position"]), change["tar_nt"])
                    elif change["tar_nt"] == "-":
                        seq_modifier.remove(int(change["position"]),
                                            len(change["ref_nt"]))
                    else:
                        seq_modifier.replace(
                                     int(change["position"]), change["tar_nt"])
                record = SeqRecord(Seq(seq_modifier.seq()))
                record.id = data["target_id"]
                record.description = ""
                SeqIO.write(record, os.path.join(
                            output_folder, record.id + ".fa"), "fasta")

    def modify_header(self, input_file):
        first = True
        tmp_file_path = input_file + "_TMP"
        output_fh = open(input_file + "_TMP", "w")
        with open(input_file, "r") as s_h:
            for line in s_h:
                line = line.strip()
                if first:
                    first = False
                    if (line[0] != ">"):
                        print("Error: No proper header!!")
                        sys.exit()
                if line.startswith(">"):
                    mod = line.split("|")
                    folder = input_file.split("/")
                    folder = "/".join(folder[:-1])
                    if (len(mod) == 5) and (line[0] == ">"):
                        new_header = ">%s" % (mod[3])
                    elif (len(mod) != 5) and (line[0] == ">"):
                        new_header = line.split(" ")[0]
                    elif (line[0] != ">"):
                        print("Error: No proper header!!")
                        sys.exit()
                    line = new_header
                output_fh.write(line + "\n")
        output_fh.close()
        shutil.move(tmp_file_path, input_file)



import copy
import numpy as np
from annogesiclib.helper import Helper
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.lib_reader import read_libs, read_wig
from annogesiclib.coverage_detection import replicate_comparison, get_repmatch


def check_start_and_end(start, end, covers):
    if (start - 2) < 0:
        c_start = 0
    else:
        c_start = start - 2
    if (end + 2) > len(covers):
        c_end = len(covers)
    else:
        c_end = end + 2
    return c_start, c_end


def get_coverage(sorf, wigs, strand, coverages, medianlist, cutoffs):
    high_cover = -1
    low_cover = -1
    sorf_covers = {}
    for wig_strain, conds in wigs.items():
        if wig_strain == sorf["strain"]:
            for cond, tracks in conds.items():
                sorf_covers[cond] = []
                for lib_name, covers in tracks.items():
                    track = lib_name.split("|")[-3]
                    lib_strand = lib_name.split("|")[-2]
                    lib_type = lib_name.split("|")[-1]
                    total_cover = 0
                    first = True
                    c_start, c_end = check_start_and_end(
                            sorf["start"], sorf["end"], covers)
                    covers = covers[c_start: c_end]
                    if strand == "-":
                        covers = covers[::-1]
                    pos = 0
                    for cover in covers:
                        if (lib_strand == strand):
                            if strand == "+":
                                cover_pos = c_start + pos
                            else:
                                cover_pos = c_end - pos
                            if (sorf["start"] <= cover_pos) and (
                                    sorf["end"] >= cover_pos):
                                total_cover = total_cover + cover
                                if first:
                                    first = False
                                    high_cover = cover
                                    low_cover = cover
                                else:
                                    if high_cover < cover:
                                        high_cover = cover
                                    if low_cover > cover:
                                        low_cover = cover
                        pos += 1
                    avg = total_cover / float(sorf["end"] - sorf["start"] + 1)
                    if medianlist is not None:
                        cutoff_cover = get_cutoff(sorf, track,
                                                  coverages, medianlist)
                    else:
                        cutoff_cover = coverages
                    if cutoffs is not None:
                        cutoffs[track] = cutoff_cover
                    if avg > float(cutoff_cover):
                        sorf_covers[cond].append({"track": track,
                                                  "high": high_cover,
                                                  "low": low_cover, "avg": avg,
                                                  "type": lib_type,
                                                  "pos": sorf["start"]})
    return sorf_covers


def import_sorf(inter, sorfs, start, end, type_, fasta, rbs):
    sorfs.append({"strain": inter.seq_id,
                  "strand": inter.strand,
                  "start": start,
                  "end": end,
                  "starts": [str(start)],
                  "ends": [str(end)],
                  "seq": fasta[start-1:end],
                  "type": type_,
                  "print": False,
                  "rbs": rbs})


def detect_rbs_site(fasta, start, inter, args_sorf):
    '''detect the ribosome binding site'''
    detect = []
    pre_miss = 5
    for nts in range(0, start):
        num = 0
        miss = 0
        for nt in fasta[nts:nts + 6]:
            if miss > args_sorf.fuzzy_rbs:
                break
            else:
                if (num == 0) and (nt != "A"):
                    miss += 1
                elif (num == 1) and (nt != "G"):
                    miss += 1
                elif (num == 2) and (nt != "G"):
                    miss += 1
                elif (num == 3) and (nt != "A"):
                    miss += 1
                elif (num == 4) and (nt != "G"):
                    miss += 1
                elif (num == 5) and (nt != "G"):
                    miss += 1
                num += 1
        if miss <= args_sorf.fuzzy_rbs:
            if (miss <= pre_miss):
                if miss < pre_miss:
                    detect = []
                if inter.strand == "+":
                    detect.append(inter.start + nts)
                else:
                    detect.append(inter.start + (len(fasta) - nts) - 1)
                pre_miss = miss
    if len(detect) == 0:
        detect = ["NA"]
    return detect


def check_terminal_seq(seq, start, end, args_sorf, source, inter, sorfs, rbs):
    '''check the sequence which are located at the two ends'''
    detect = None
    for i in [0, 1, -1, 2, -2]:
        fasta = Helper().extract_gene(seq, start + i, end + i, inter.strand)
        if (fasta[:3] in args_sorf.start_codon) and (
                fasta[-3:] in args_sorf.stop_codon):
            detect = i
    if detect is not None:
        start = start + detect
        end = end + detect
        import_sorf(inter, sorfs, start, end, source, seq, rbs)


def detect_start_stop(inters, seq, args_sorf):
    '''check the length is 3 -times or not'''
    sorfs = []
    for inter in inters:
        fasta = Helper().extract_gene(
                seq[inter.seq_id], inter.start, inter.end, inter.strand)
        starts = []
        stops = []
        for frame in range(0, 3):
            for index in range(frame, len(fasta), 3):
                if fasta[index:index + 3] in args_sorf.start_codon:
                    starts.append(index)
                elif fasta[index:index + 3] in args_sorf.stop_codon:
                    stops.append(index)
        for start in starts:
            for stop in stops:
                if ((stop - start) > 0) and \
                   (((stop - start) % 3) == 0) and \
                   ((stop - start) <= args_sorf.max_len) and \
                   ((stop - start) >= args_sorf.min_len):
                    rbs = detect_rbs_site(fasta, start, inter, args_sorf)
                    if (len(rbs) == 1) and (rbs[0] == "NA"):
                        pass
                    else:
                        if (inter.source == "intergenic") or (
                                inter.source == "antisense"):
                            if inter.strand == "+":
                                check_terminal_seq(
                                    seq[inter.seq_id], inter.start + start,
                                    inter.start + stop + 2, args_sorf,
                                    inter.source, inter, sorfs, rbs)
                            else:
                                check_terminal_seq(
                                    seq[inter.seq_id],
                                    inter.start + (len(fasta) - stop - 3),
                                    inter.start + (len(fasta) - start - 1),
                                    args_sorf, inter.source, inter, sorfs, rbs)
                        elif inter.source == "UTR_derived":
                            if inter.strand == "+":
                                check_terminal_seq(
                                    seq[inter.seq_id], inter.start + start,
                                    inter.start + stop + 2, args_sorf,
                                    inter.attributes["UTR_type"],
                                    inter, sorfs, rbs)
                            else:
                                check_terminal_seq(
                                    seq[inter.seq_id],
                                    inter.start + (len(fasta) - stop - 3),
                                    inter.start + (len(fasta) - start - 1),
                                    args_sorf, inter.attributes["UTR_type"],
                                    inter, sorfs, rbs)
    return sorfs


def read_data(inter_gff, tss_file, srna_gff, fasta, utr_detect):
    seq = {}
    inters = []
    tsss = []
    srnas = []
    fh = open(inter_gff)
    for entry in Gff3Parser().entries(fh):
        if ((entry.source == "UTR_derived") and (
                utr_detect)) or (
                (entry.source == "intergenic") or (
                entry.source == "antisense")):
            inters.append(entry)
    inters = sorted(inters, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    fh.close()
    if tss_file is not None:
        fh = open(tss_file)
        for entry in Gff3Parser().entries(fh):
            tsss.append(entry)
        tsss = sorted(tsss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
        fh.close()
    else:
        tsss = None
    if srna_gff is not None:
        fh = open(srna_gff)
        for entry in Gff3Parser().entries(fh):
            new = {}
            for key, value in entry.attributes.items():
                if "sORF" not in key:
                    new[key] = value
            entry.attributes = copy.deepcopy(new)
            srnas.append(entry)
        srnas = sorted(srnas, key=lambda k: (k.seq_id, k.start,
                                             k.end, k.strand))
        fh.close()
    else:
        srnas = None
    with open(fasta, "r") as s_f:
        for line in s_f:
            line = line.strip()
            if line.startswith(">"):
                strain = line[1:]
                seq[strain] = ""
            else:
                seq[strain] = seq[strain] + line
    return inters, tsss, srnas, seq


def check_tss(sorf, tss, utr_fuzzy, checks):
    if ((sorf["start"] - tss.start <= utr_fuzzy) and (
         sorf["start"] - tss.start >= 0) and (sorf["strand"] == "+")) or (
        (tss.start - sorf["end"] <= utr_fuzzy) and (
         tss.start - sorf["end"] >= 0) and (sorf["strand"] == "-")):
        sorf["start_TSS"] = str(tss.start) + "_" + tss.strand
        sorf["with_TSS"].append("TSS:" + str(tss.start) + "_" + tss.strand)
        checks["start"] = True
        checks["import"] = True
        rbss = []
        if (sorf["rbs"][0] != "NA"):
            if sorf["strand"] == "+":
                for rbs in sorf["rbs"]:
                    if rbs >= tss.start:
                        rbss.append(rbs)
            else:
                for rbs in sorf["rbs"]:
                    if rbs <= tss.start:
                        rbss.append(rbs)
        if len(rbss) != 0:
            checks["rbs"] = rbss


def compare_sorf_tss(sorfs, tsss, tss_file, args_sorf):
    sorfs_all = []
    sorfs_best = []
    if tss_file is not None:
        for sorf in sorfs:
            checks = {"start": False, "rbs": False, "import": False}
            sorf["with_TSS"] = []
            for tss in tsss:
                checks["import"] = False
                if (sorf["strain"] == tss.seq_id) and (
                        sorf["strand"] == tss.strand):
                    if sorf["strand"] == "+":
                        check_tss(sorf, tss, args_sorf.utr_length, checks)
                    else:
                        check_tss(sorf, tss, args_sorf.utr_length, checks)
                    if not checks["import"]:
                        if (tss.start <= sorf["start"]) and (
                                tss.start >= sorf["end"]):
                            sorf["with_TSS"].append("TSS_" + str(
                                                    tss.start) + tss.strand)
            if not checks["start"]:
                sorf["start_TSS"] = "NA"
            if len(sorf["with_TSS"]) == 0:
                sorf["with_TSS"] = ["NA"]
            if (checks["rbs"] and (not args_sorf.noafter_tss) and (
                    not args_sorf.no_tss)):
                sorf["rbs"] = checks["rbs"]
                sorfs_best.append(copy.deepcopy(sorf))
            elif ((sorf["rbs"][0] != "NA") and (args_sorf.noafter_tss) and (
                    not args_sorf.no_tss) and (checks["start"])):
                sorfs_best.append(copy.deepcopy(sorf))
            elif ((sorf["rbs"][0] != "NA") and (args_sorf.noafter_tss) and (
                    args_sorf.no_tss)):
                sorfs_best.append(copy.deepcopy(sorf))
            sorfs_all.append(sorf)
    else:
        for sorf in sorfs:
            sorf["with_TSS"] = ["NA"]
            sorf["start_TSS"] = "NA"
            if sorf["rbs"][0] != "NA":
                sorfs_best.append(copy.deepcopy(sorf))
            sorfs_all.append(sorf)
    return sorfs_all, sorfs_best


def compare_sorf_srna(sorfs, srnas, srna_gff):
    if srna_gff is not None:
        for sorf in sorfs:
            sorf["srna"] = []
            for srna in srnas:
                if (sorf["strain"] == srna.seq_id) and (
                        sorf["strand"] == srna.strand):
                    if ((srna.start <= sorf["start"]) and (
                             srna.end >= sorf["end"])) or (
                            (srna.start >= sorf["start"]) and (
                             srna.end <= sorf["end"])) or (
                            (srna.start <= sorf["start"]) and (
                             srna.end >= sorf["start"]) and (
                             srna.end <= sorf["end"])) or (
                            (srna.start >= sorf["start"]) and (
                             srna.start <= sorf["end"]) and (
                             srna.end >= sorf["end"])):
                        sorf["srna"].append("sRNA:" +
                                            str(srna.start) + "-" +
                                            str(srna.end) + "_" + srna.strand)
            if len(sorf["srna"]) == 0:
                sorf["srna"] = ["NA"]
    else:
        for sorf in sorfs:
            sorf["srna"] = ["NA"]


def import_overlap(sorf2, final, sorf1, first):
    sorf2["print"] = True
    if final["start"] > sorf2["start"]:
        final["start"] = sorf2["start"]
    if final["end"] < sorf2["end"]:
        final["end"] = sorf2["end"]
    if first:
        final["candidate"] = []
        final["candidate"].append("_".join(["-".join([
                str(sorf1["start"]), str(sorf1["end"])]),
                "TSS:" + sorf1["start_TSS"],
                "RBS:" + str(sorf1["rbs"][0])]))
        first = False
    if "_".join(["-".join([str(sorf2["start"]), str(sorf2["end"])]),
                 "TSS:" + sorf2["start_TSS"],
                 "RBS:" + str(sorf2["rbs"][0])]) not in final["candidate"]:
        final["candidate"].append("_".join(["-".join([
                str(sorf2["start"]), str(sorf2["end"])]),
                "TSS:" + sorf2["start_TSS"],
                "RBS:" + str(sorf2["rbs"][0])]))
    if str(sorf2["start"]) not in final["starts"]:
        final["starts"].append(str(sorf2["start"]))
    if str(sorf2["end"]) not in final["ends"]:
        final["ends"].append(str(sorf2["end"]))
    if sorf2["rbs"] != ["NA"]:
        if (len(final["rbs"]) == 1) and (final["rbs"] == ["NA"]):
            final["rbs"] = sorf2["rbs"]
        else:
            if sorf2["rbs"][0] not in final["rbs"]:
                final["rbs"] = final["rbs"] + sorf2["rbs"]
    return first


def merge(sorfs, seq):
    '''merge the overlapped sORF'''
    finals = []
    for sorf1 in sorfs:
        final = copy.deepcopy(sorf1)
        first = True
        if not sorf1["print"]:
            sorf1["print"] = True
            for sorf2 in sorfs:
                overlap = False
                if (final["strain"] == sorf2["strain"]) and (
                        final["strand"] == sorf2["strand"]) and (
                        final["rbs"] == sorf2["rbs"]):
                    if (final["start"] >= sorf2["start"]) and (
                            final["end"] <= sorf2["end"]):
                        overlap = True
                    elif (final["start"] >= sorf2["start"]) and (
                            final["start"] <= sorf2["end"]) and (
                            final["end"] >= sorf2["end"]):
                        overlap = True
                    elif (final["start"] <= sorf2["start"]) and (
                            final["end"] >= sorf2["start"]) and (
                            final["end"] <= sorf2["end"]):
                        overlap = True
                    elif (final["start"] <= sorf2["start"]) and (
                            final["end"] >= sorf2["end"]):
                        overlap = True
                    elif (sorf2["start"] > final["end"]):
                        break
                    if overlap:
                        first = import_overlap(sorf2, final, sorf1, first)
            final["seq"] = Helper().extract_gene(
                    seq[final["strain"]], final["start"],
                    final["end"], final["strand"])
            new = {}
            for key, value in final.items():
                if "print" not in key:
                    new[key] = value
            final = copy.deepcopy(new)
            finals.append(final)
    return finals


def assign_utr_cutoff(coverages, utr_type, medians):
    if coverages[utr_type] == "median":
        cutoff = medians["median"]
    elif coverages[utr_type] == "mean":
        cutoff = medians["mean"]
    else:
        cutoff = float(coverages[utr_type])
    return cutoff


def get_cutoff(sorf, track, coverages, medians):
    if sorf["type"] == "intergenic":
        cutoff_cover = float(coverages["inter"])
    elif sorf["type"] == "antisense":
        cutoff_cover = float(coverages["anti"])
    elif ("5utr" in sorf["type"]) and ("3utr" in sorf["type"]):
        cutoff_utr3 = assign_utr_cutoff(
                coverages, "3utr", medians[sorf["strain"]]["3utr"][track])
        cutoff_utr5 = assign_utr_cutoff(
                coverages, "5utr", medians[sorf["strain"]]["5utr"][track])
        cutoff_cover = min(cutoff_utr5, cutoff_utr3)
    elif ("5utr" in sorf["type"]):
        cutoff_cover = assign_utr_cutoff(
                coverages, "5utr", medians[sorf["strain"]]["5utr"][track])
    elif ("3utr" in sorf["type"]):
        cutoff_cover = assign_utr_cutoff(
                coverages, "3utr", medians[sorf["strain"]]["3utr"][track])
    elif ("interCDS" in sorf["type"]):
        cutoff_cover = assign_utr_cutoff(
                coverages, "interCDS",
                medians[sorf["strain"]]["interCDS"][track])
    return cutoff_cover


def get_attribute(num, name, start_tss, sorf, type_):
    if (type_ == "intergenic") or (type_ == "intergenic"):
        attribute_string = ";".join(
            ["=".join(items) for items in (
                ["ID", sorf["strain"] + "_sorf" + str(num)],
                ["Name", "sORF_" + name],
                ["start_TSS", start_tss],
                ["with_TSS", ",".join(sorf["with_TSS"])],
                ["sRNA", ",".join(sorf["srna"])],
                ["rbs", ",".join(sorf["rbs"])],
                ["frame_shift", str(sorf["shift"])],
                ["sORF_type", type_])])
    else:
        attribute_string = ";".join(
            ["=".join(items) for items in (
                ["ID", sorf["strain"] + "_sorf" + str(num)],
                ["Name", "sORF_" + name],
                ["start_TSS", start_tss],
                ["with_TSS", ",".join(sorf["with_TSS"])],
                ["sORF_type", sorf["type"]],
                ["sRNA", ",".join(sorf["srna"])],
                ["rbs", ",".join(sorf["rbs"])],
                ["frame_shift", str(sorf["shift"])])])
    return attribute_string


def check_start_and_tss_point(sorf):
    '''searching the associated TSS'''
    tsss = []
    for tss in sorf["with_TSS"]:
        if tss != "NA":
            if (int(tss.replace("TSS:", "")[:-2]) >= int(sorf["start"])) and (
                    int(tss.replace("TSS:", "")[:-2]) <= int(sorf["end"])):
                tsss.append(tss)
        else:
            tsss.append(tss)
    sorf["with_TSS"] = copy.deepcopy(tsss)


def compare_rbs_start(sorf, min_rbs, max_rbs):
    '''searching the associated ribosome binding site'''
    detect = False
    if (len(sorf["rbs"]) == 1) and (sorf["rbs"][0] == "NA"):
        pass
    else:
        new_rbss = []
        for rbs in sorf["rbs"]:
            if rbs != "NA":
                if sorf["strand"] == "+":
                    for start in sorf["starts"]:
                        if ((int(start) - int(rbs)) >= min_rbs + 6) and (
                                (int(start) - int(rbs)) <= max_rbs + 6) and (
                                rbs not in new_rbss):
                            new_rbss.append(rbs)
                            detect = True
                else:
                    for end in sorf["ends"]:
                        if ((int(rbs) - int(end)) >= min_rbs + 6) and (
                                (int(rbs) - int(end)) <= max_rbs + 6) and (
                                rbs not in new_rbss):
                            new_rbss.append(rbs)
                            detect = True
        if not detect:
            sorf["rbs"] = ["NA"]
        else:
            sorf["rbs"] = new_rbss
    return detect


def gen_new_candidates(sorf, min_rbs, max_rbs):
    new_candidates = []
    for start in sorf["starts"]:
        for end in sorf["ends"]:
            if ((int(end) - int(start) + 1) % 3) == 0:
                for rbs in sorf["rbs"]:
                    if (sorf["strand"] == "+") and (
                            (int(start) - rbs) >= (min_rbs + 6)) and (
                            (int(start) - rbs) <= (max_rbs + 6)):
                        if sorf["with_TSS"] == ["NA"]:
                            new_candidates.append("_".join(["-".join([
                                    start, end]), "NA",
                                    "RBS:" + str(rbs)]))
                        else:
                            for tss in sorf["with_TSS"]:
                                if int(tss.split(":")[-1][:-2]) > int(start):
                                    break
                                pre_tss = tss
                            new_candidates.append("_".join(["-".join([
                                  start, end]), pre_tss.replace("TSS_", "TSS:"),
                                  "RBS:" + str(rbs)]))
                    elif (sorf["strand"] == "-") and (
                            (rbs - int(end)) >= (min_rbs + 6)) and (
                            (rbs - int(end)) <= (max_rbs + 6)):
                        if sorf["with_TSS"] == ["NA"]:
                            new_candidates.append("_".join(["-".join([
                                    start, end]), "NA",
                                    "RBS:" + str(rbs)]))
                        else:
                            for tss in sorf["with_TSS"]:
                                if int(tss.split(":")[-1][:-2]) <= int(start):
                                    break
                            new_candidates.append("_".join(["-".join([
                                  start, end]), tss.replace("TSS_", "TSS:"),
                                  "RBS:" + str(rbs)]))
    return new_candidates


def check_candidates_srnas(sorf, min_rbs, max_rbs):
    '''assign the sRNA which overlap with sORF to corresponding candidates'''
    new_candidates = []
    for cand in sorf["candidate"]:
        infos = cand.split("_")
        start = infos[0].split("-")[0]
        end = infos[0].split("-")[1]
        rbs = int(infos[-1].split(":")[-1])
        if (start in sorf["starts"]) and (
                end in sorf["ends"]) and (
                rbs in sorf["rbs"]):
            new_candidates.append(cand)
    if len(new_candidates) == 0:
        new_candidates = gen_new_candidates(sorf, min_rbs, max_rbs)
    new_srnas = []
    if (len(sorf["srna"]) == 1) and (sorf["srna"][0] == "NA"):
        pass
    else:
        for srna in sorf["srna"]:
            srna_strand = srna.split("_")[-1]
            if srna_strand == "r":
                strand = "-"
            else:
                strand = "+"
            srna_end = int(srna.split("_")[-2].split("-")[-1])
            srna_start = int(srna.split("_")[-2].split("-")[0].split(":")[-1])
            if (strand == sorf["strand"]):
                if ((srna_start <= int(sorf["start"])) and (
                         srna_end >= int(sorf["end"]))) or (
                        (srna_start >= int(sorf["start"])) and (
                         srna_end <= int(sorf["end"]))) or (
                        (srna_start <= int(sorf["start"])) and (
                         srna_end >= int(sorf["start"])) and (
                         srna_end <= int(sorf["end"]))) or (
                        (srna_start >= int(sorf["start"])) and (
                         srna_start <= int(sorf["end"])) and (
                         srna_end >= int(sorf["end"]))):
                    new_srnas.append(srna)
    sorf["candidate"] = new_candidates
    if len(new_srnas) != 0:
        sorf["srna"] = new_srnas
    else:
        sorf["srna"] = ["NA"]


def assign_sorf(sorf, starts, ends, fasta):
    sorf["starts"] = starts
    sorf["ends"] = ends
    sorf["start"] = min(map(int, starts))
    sorf["end"] = max(map(int, ends))
    sorf["seq"] = Helper().extract_gene(
            fasta[sorf["strain"]], sorf["start"],
            sorf["end"], sorf["strand"])


def check_start_end(sorf, args_sorf, fasta):
    '''check the start and end point which can form proper protein 
    or not (3 times)'''
    if (len(sorf["rbs"]) == 1) and (sorf["rbs"][0] == "NA"):
        pass
    else:
        if sorf["strand"] == "+":
            starts = []
            for start in sorf["starts"]:
                if int(start) < min(sorf["rbs"]):
                    continue
                else:
                    for rbs in sorf["rbs"]:
                        if ((int(start) - int(rbs)) >=
                                args_sorf.min_rbs + 6) and (
                                (int(start) - int(rbs)) <=
                                args_sorf.max_rbs + 6) and (
                                start not in starts):
                            starts.append(start)
            ends = []
            for end in sorf["ends"]:
                for start in starts:
                    if ((int(end) - int(start) + 1) % 3 == 0) and (
                            (int(end) - int(start) + 1) >=
                            args_sorf.min_len) and (
                            (int(end) - int(start) + 1) <=
                            args_sorf.max_len) and (
                            end not in ends):
                        if end not in ends:
                            ends.append(end)
        else:
            ends = []
            for end in sorf["ends"]:
                if int(end) > max(sorf["rbs"]):
                    continue
                else:
                    for rbs in sorf["rbs"]:
                        if ((int(rbs) - int(end)) >=
                                args_sorf.min_rbs + 6) and (
                                (int(rbs) - int(end)) <=
                                args_sorf.max_rbs + 6) and (
                                end not in ends):
                            ends.append(end)
            starts = []
            for start in sorf["starts"]:
                for end in ends:
                    if ((int(end) - int(start) + 1) % 3 == 0) and (
                            (int(end) - int(start) + 1) >=
                            args_sorf.min_len) and (
                            (int(end) - int(start) + 1) <=
                            args_sorf.max_len) and (
                            start not in starts):
                        if start not in starts:
                            starts.append(start)
        assign_sorf(sorf, starts, ends, fasta)
        check_candidates_srnas(sorf, args_sorf.min_rbs, args_sorf.max_rbs)


def detect_frame_shift(sorf):
    '''check the frame shift'''
    stand = sorf["starts"][0]
    shift = {"0": False, "1": False, "2": False}
    sorf["shift"] = 0
    for start in sorf["starts"]:
        if ((int(start) - int(stand)) % 3) == 0:
            shift["0"] = True
        elif ((int(start) - int(stand)) % 3) == 1:
            shift["1"] = True
        elif ((int(start) - int(stand)) % 3) == 2:
            shift["2"] = True
    for key, value in shift.items():
        if value:
            sorf["shift"] += 1


def print_file(sorf, sorf_datas, num, out_g, out_t, file_type, args_sorf):
    name = '%0*d' % (5, num)
    if (sorf["type"] == "intergenic") or (sorf["type"] == "antisense"):
        if (sorf["type"] == "intergenic"):
            type_ = "Intergenic"
        else:
            type_ = "Antisense"
        for index in range(len(sorf["rbs"])):
            if (sorf["rbs"][index] == "NA") and (len(sorf["rbs"]) == 1):
                pass
            else:
                sorf["rbs"][index] = "RBS_" + str(sorf["rbs"][index])
        attribute_string = get_attribute(num, name, sorf["start_TSS"],
                                         sorf, type_.lower())
    else:
        if ("3utr" in sorf["type"]) and ("5utr" in sorf["type"]):
            type_ = "3'UTR_derived;5'UTR_derived"
        elif ("3utr" in sorf["type"]):
            type_ = "3'UTR_derived"
        elif ("5utr" in sorf["type"]):
            type_ = "5'UTR_derived"
        elif ("interCDS" in sorf["type"]):
            type_ = "interCDS"
        for index in range(len(sorf["rbs"])):
            if (sorf["rbs"][index] == "NA") and (len(sorf["rbs"]) == 1):
                pass
            else:
                sorf["rbs"][index] = "RBS_" + str(sorf["rbs"][index])
        attribute_string = get_attribute(num, name, sorf["start_TSS"],
                                         sorf, "utr")
    info = "\t".join([str(field) for field in [
                      sorf["strain"], "ANNOgesic", "sORF", str(sorf["start"]),
                      str(sorf["end"]), ".", sorf["strand"],
                      ".", attribute_string]])
    out_g.write(info + "\n")
    if ("frag" in ";".join(sorf_datas["conds"].keys())) and (
            "tex" in ";".join(sorf_datas["conds"].keys())):
        lib_type = "TEX+/-;Fragmented"
    elif ("frag" in ";".join(sorf_datas["conds"].keys())):
        lib_type = "Fragmented"
    elif ("tex" in ";".join(sorf_datas["conds"].keys())):
        lib_type = "TEX+/-"
    print_table(out_t, sorf, name, type_, lib_type,
                sorf_datas, args_sorf)


def print_table(out_t, sorf, name, type_, lib_type, sorf_datas, args_sorf):
    out_t.write("\t".join([sorf["strain"], "sORF_" + name, str(sorf["start"]),
                           str(sorf["end"]), sorf["strand"], type_,
                           ";".join(sorf["with_TSS"]), ";".join(sorf["rbs"]),
                           ";".join(sorf["starts"]), ";".join(sorf["ends"]),
                           ";".join(sorf["srna"]), str(sorf["shift"]),
                           lib_type, str(sorf_datas["best"]),
                           str(sorf_datas["high"]),
                           str(sorf_datas["low"])]) + "\t")
    if not args_sorf.table_best:
        first = True
        for data in sorf_datas["detail"]:
            if first:
                out_t.write("{0}(avg={1};high={2};low={3})".format(
                            data["track"], data["avg"],
                            data["high"], data["low"]))
                first = False
            else:
                out_t.write(";{0}(avg={1};high={2};low={3})".format(
                            data["track"], data["avg"],
                            data["high"], data["low"]))
    else:
        out_t.write("{0}(avg={1};high={2};low={3})".format(
                    sorf_datas["track"], sorf_datas["best"],
                    sorf_datas["high"], sorf_datas["low"]))
    out_t.write("\t" + sorf["seq"])
    if args_sorf.print_all:
        out_t.write("\t" + ";".join(sorf["candidate"]))
    out_t.write("\n")


def get_inter_coverage(inters, inter_covers):
    for datas in inters:
        for cond, covers in datas.items():
            for inter in covers:
                if inter["track"] not in inter_covers.keys():
                    inter_covers[inter["track"]] = []
                inter_covers[inter["track"]].append(inter["avg"])


def detect_utr_type(inter, utr_type, med_inters, wigs, strand, background):
    '''detect the type of UTR-derived sORF'''
    if inter.attributes["UTR_type"] == utr_type:
        inter_datas = {}
        inter_datas["strain"] = inter.seq_id
        inter_datas["strand"] = inter.strand
        inter_datas["start"] = inter.start
        inter_datas["end"] = inter.end
        inter_datas = get_coverage(inter_datas, wigs,
                                   strand, background, None, None)
        med_inters[inter.seq_id][utr_type].append(inter_datas)


def median_score(lst, cutoff):
    '''If the cutoff is assigned by percentage, 
    it will get the corresponding number'''
    if "p_" in cutoff:
        per = float(cutoff.split("_")[-1])
        sortedLst = sorted(lst)
        lstLen = len(lst)
        index = int((lstLen - 1) * per)
        if lstLen != 0:
            return sortedLst[index]
        else:
            return 0
    else:
        return cutoff


def mean_score(lst):
    total = 0
    for li in lst:
        total = total + li
    if len(lst) != 0:
        return (total / len(lst))
    else:
        return 0


def validate_tss(starts, ends, sorf, utr_fuzzy):
    '''compare sORF with TSS'''
    tsss = []
    start_pos = "NA"
    if sorf["with_TSS"][0] != "NA":
        for tss in sorf["with_TSS"]:
            tss_start = int(tss.replace("TSS:", "")[:-2])
            if sorf["strand"] == "+":
                if (tss_start >= min(starts) - utr_fuzzy) and (
                        tss_start <= max(ends)):
                    tsss.append(tss)
                    if (tss_start >= min(starts) - utr_fuzzy) and (
                            tss_start <= min(starts)):
                        start_pos = tss
            else:
                if (tss_start >= min(starts)) and (
                        tss_start <= max(ends) + utr_fuzzy):
                    tsss.append(tss)
                    if (tss_start <= min(ends) + utr_fuzzy) and (
                            tss_start >= min(ends)):
                        start_pos = tss
                        break
    else:
        tsss = ["NA"]
    return (tsss, start_pos)


def validate_srna(starts, ends, sorf):
    '''compare sORF with sRNA'''
    srnas = []
    for srna in sorf["srna"]:
        if srna == "NA":
            break
        else:
            datas = srna.split(":")[1][:-2].split("-")
            start = int(datas[0])
            end = int(datas[1])
            for index in range(0, len(starts)):
                if ((start <= starts[index]) and (
                         end >= ends[index])) or (
                        (start >= starts[index]) and (
                         end <= ends[index])) or (
                        (start >= starts[index]) and (
                         start <= ends[index]) and (
                         end >= ends[index])) or (
                        (start <= starts[index]) and (
                         end >= starts[index]) and (
                         end <= ends[index])):
                    srnas.append(srna)
                    break
    if len(srnas) == 0:
        srnas = ["NA"]
    return srnas


def get_best(sorfs, tss_file, srna_file, args_sorf):
    '''based on the filers to get the best results'''
    final_sorfs = []
    for sorf in sorfs:
        if (tss_file is not None):
            if (sorf["with_TSS"][0] != "NA") or (args_sorf.no_tss):
                cands = []
                starts = []
                ends = []
                tmp_sorf = copy.deepcopy(sorf)
                for candidate in sorf["candidate"]:
                    tss = candidate.split("_TSS:")[1].split("_RBS:")[0]
                    rbs = candidate.split("_TSS:")[1].split("_RBS:")[-1]
                    if (tss != "NA") or (args_sorf.no_tss):
                        datas = candidate.split("_TSS:")[0].split("-")
                        cands.append("-".join([
                              str(datas[0]), str(datas[1])]) + "_TSS:" +
                              tss + "_RBS:" + rbs)
                        starts.append(int(datas[0]))
                        ends.append(int(datas[1]))
                tmp_sorf["start"] = min(starts)
                tmp_sorf["end"] = max(ends)
                tmp_sorf["starts"] = sorf["starts"]
                tmp_sorf["ends"] = sorf["ends"]
                tmp_sorf["candidate"] = cands
                tsss_datas = validate_tss(starts, ends, sorf,
                                          args_sorf.utr_length)
                tmp_sorf["with_TSS"] = tsss_datas[0]
                tmp_sorf["start_TSS"] = tsss_datas[1]
                if srna_file is not None:
                    tmp_sorf["sRNA"] = validate_srna(starts, ends, sorf)
                    if (args_sorf.no_srna) and (tmp_sorf["sRNA"][0] == "NA"):
                        final_sorfs.append(tmp_sorf)
                    elif not args_sorf.no_srna:
                        final_sorfs.append(tmp_sorf)
        elif srna_file is not None:
            if (args_sorf.no_srna) and (tmp_sorf["sRNA"][0] == "NA"):
                final_sorfs.append(sorf)
            elif not args_sorf.no_srna:
                    final_sorfs.append(tmp_sorf)
    if len(final_sorfs) == 0:
        final_sorfs = sorfs
    return final_sorfs


def coverage_and_output(sorfs, mediandict, wigs, out_g, out_t,
                        file_type, fasta, coverages, args_sorf, texs):
    '''get the coverage of sORF and print it out'''
    out_g.write("##gff-version 3\n")
    if args_sorf.print_all:
        out_t.write("\t".join([
            "Genome", "Name", "Start", "End", "Strand", "Type", "TSS",
            "Ribosome_binding_site", "All_start_points", "All_stop_points",
            "sRNA_conflict", "Frame_shift", "Lib_type", "Best_avg_coverage",
            "Best_highest_coverage", "Best_lowest_coverage", "Track_detail",
            "Seq", "Combinations"]) + "\n")
    else:
        out_t.write("\t".join([
            "Genome", "Name", "Start", "End", "Strand", "Type", "TSS",
            "Ribosome_binding_site", "All_start_points", "All_stop_points",
            "sRNA_conflict", "Frame_shift", "Lib_type", "Best_avg_coverage",
            "Best_highest_coverage", "Best_lowest_coverage",
            "Track_detail", "Seq"]) + "\n")
    num = 0
    for sorf in sorfs:
        if ((compare_rbs_start(sorf, args_sorf.min_rbs,
                               args_sorf.max_rbs)) and (
                file_type == "best")) or (
                file_type == "all"):
            if file_type == "best":
                check_start_end(sorf, args_sorf, fasta)
            detect_frame_shift(sorf)
            cutoffs = {}
            if sorf["strand"] == "+":
                sorf_covers = get_coverage(sorf, wigs["forward"], "+",
                                           coverages, mediandict, cutoffs)
            else:
                sorf_covers = get_coverage(sorf, wigs["reverse"], "-",
                                           coverages, mediandict, cutoffs)
            if len(sorf_covers) != 0:
                sorf_info = replicate_comparison(
                        args_sorf, sorf_covers, sorf["strand"], "sORF", None,
                        cutoffs, None, cutoffs, None, texs)
                if len(sorf_info["conds"].keys()) != 0:
                    print_file(sorf, sorf_info, num, out_g, out_t, file_type,
                               args_sorf)
                    num += 1


def detect_inter_type(inters, wigs, background):
    '''detect the types of intergenic sORF'''
    med_inters = {}
    strain = ""
    for inter in inters:
        if inter.seq_id != strain:
            strain = inter.seq_id
            med_inters[inter.seq_id] = {"5utr": [], "3utr": [], "interCDS": []}
        if (inter.source == "UTR_derived") and (inter.strand == "+"):
            detect_utr_type(inter, "5utr", med_inters,
                            wigs["forward"], "+", background)
            detect_utr_type(inter, "3utr", med_inters,
                            wigs["forward"], "+", background)
            detect_utr_type(inter, "interCDS", med_inters,
                            wigs["forward"], "+", background)
        elif (inter.source == "UTR_derived") and (inter.strand == "-"):
            detect_utr_type(inter, "5utr", med_inters,
                            wigs["reverse"], "-", background)
            detect_utr_type(inter, "3utr", med_inters,
                            wigs["reverse"], "-", background)
            detect_utr_type(inter, "interCDS", med_inters,
                            wigs["reverse"], "-", background)
    return med_inters


def set_median(covers, mediandict, coverages):
    for strain, utrs in covers.items():
        mediandict[strain] = {"3utr": {}, "5utr": {}, "interCDS": {}}
        for utr, tracks in utrs.items():
            for track, avgs in tracks.items():
                if track not in mediandict[strain][utr].keys():
                    mediandict[strain][utr][track] = {}
                mediandict[strain][utr][track] = {"median": median_score(
                                                      avgs, coverages[utr])}
    for utr, value in coverages.items():
        if type(value) is str:
            if "p_" in value:
                coverages[utr] = "median"


def compute_candidate_best(sorfs_best):
    for sorf in sorfs_best:
        sorf["candidate"] = []
        sorf["candidate"].append("_".join(["-".join([
                          str(sorf["start"]), str(sorf["end"])]),
                          "TSS:" + sorf["start_TSS"],
                          "RBS:" + str(sorf["rbs"][0])]))


def set_coverage(args_sorf):
    '''set the cutoff based on different types'''
    if "n_" in args_sorf.cutoff_3utr:
        args_sorf.cutoff_3utr = float(
            args_sorf.cutoff_3utr.split("_")[-1])
    if "n_" in args_sorf.cutoff_5utr:
        args_sorf.cutoff_5utr = float(
            args_sorf.cutoff_5utr.split("_")[-1])
    if "n_" in args_sorf.cutoff_intercds:
        args_sorf.cutoff_intercds = float(
            args_sorf.cutoff_intercds.split("_")[-1])
    coverages = {"3utr": args_sorf.cutoff_3utr,
                 "5utr": args_sorf.cutoff_5utr,
                 "inter": args_sorf.cutoff_inter,
                 "interCDS": args_sorf.cutoff_intercds,
                 "anti": args_sorf.cutoff_anti}
    return coverages


def sorf_detection(fasta, srna_gff, inter_gff, tss_file, wig_f_file,
                   wig_r_file, out_prefix, args_sorf):
    coverages = set_coverage(args_sorf)
    libs, texs = read_libs(args_sorf.libs, args_sorf.merge_wigs)
    inters, tsss, srnas, seq = read_data(inter_gff, tss_file, srna_gff,
                                         fasta, args_sorf.utr_detect)
    wigs = {"forward": read_wig(wig_f_file, "+", libs),
            "reverse": read_wig(wig_r_file, "-", libs)}
    med_inters = detect_inter_type(inters, wigs, args_sorf.background)
    inter_covers = {}
    mediandict = {}
    for strain, meds in med_inters.items():
        inter_covers[strain] = {"5utr": {}, "3utr": {}, "interCDS": {}}
        for type_, covers in meds.items():
            get_inter_coverage(covers, inter_covers[strain][type_])
    set_median(inter_covers, mediandict, coverages)
    out_ag = open("_".join([out_prefix, "all.gff"]), "w")
    out_at = open("_".join([out_prefix, "all.csv"]), "w")
    out_bg = open("_".join([out_prefix, "best.gff"]), "w")
    out_bt = open("_".join([out_prefix, "best.csv"]), "w")
    sorfs = detect_start_stop(inters, seq, args_sorf)
    sorfs_all, sorfs_best = compare_sorf_tss(sorfs, tsss, tss_file, args_sorf)
    compare_sorf_srna(sorfs_all, srnas, srna_gff)
    compare_sorf_srna(sorfs_best, srnas, srna_gff)
    sorfs_all = sorted(sorfs_all, key=lambda k: (k["strain"], k["start"],
                                                 k["end"], k["strand"]))
    sorfs_best = sorted(sorfs_best, key=lambda k: (k["strain"], k["start"],
                                                   k["end"], k["strand"]))
    sorfs_all = merge(sorfs_all, seq)
    sorfs_best = merge(sorfs_best, seq)
    final_best = get_best(sorfs_best, tss_file, srna_gff, args_sorf)
    coverage_and_output(sorfs_all, mediandict, wigs, out_ag, out_at,
                        "all", seq, coverages, args_sorf, texs)
    coverage_and_output(final_best, mediandict, wigs, out_bg, out_bt,
                        "best", seq, coverages, args_sorf, texs)
    out_ag.close()
    out_at.close()
    out_bg.close()
    out_bt.close()



import csv
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
plt.style.use('ggplot')


def plot(subs, total, unknown, strain, prefix_name):
    nums = []
    nums_no_unknown = []
    classes = []
    classes_no_unknown = []
    width = 0.4
    tmp_unknown = ["Unknown", 0]
    sort_subs = sorted(subs.items(),
                       key=lambda x: (x[1]), reverse=True)
    for datas in sort_subs:
        if datas[0] == "Unknown":
            tmp_unknown = datas
        else:
            nums.append(datas[1])
            nums_no_unknown.append(datas[1])
            classes.append(datas[0])
            classes_no_unknown.append(datas[0])
    nums.append(tmp_unknown[1])
    classes.append(tmp_unknown[0])
    plt.figure(figsize=(12, 16))
    plt.subplot(211)
    ind = np.arange(len(nums))
    plt.bar(ind, nums, width, color='#FF9999')
    plt.title('Subcellular localization including Unknown\n', fontsize=24)
    plt.ylabel('Amount', fontsize=20)
    plt.yticks(fontsize=16)
    plt.xlim([0, len(nums) + 1])
    plt.xticks(ind+width, classes, rotation=40, fontsize=20, ha='right')
    plt.tight_layout(2, None, None, None)
    plt.subplot(212)
    ind = np.arange(len(nums_no_unknown))
    plt.bar(ind, nums_no_unknown, width, color='#FF9999')
    plt.title('Subcellular localization excluding Unknown\n', fontsize=24)
    plt.ylabel('Amount', fontsize=20)
    plt.xlim([0, len(nums_no_unknown) + 1])
    plt.xticks(ind+width, classes_no_unknown, rotation=40,
               fontsize=20, ha='right')
    plt.yticks(fontsize=16)
    plt.tight_layout(2, None, None, None)
    plt.savefig("_".join([prefix_name, strain, "sublocal.png"]))


def read_table(psortb_file):
    subs = {}
    subs["all_genome"] = {}
    total_nums = {}
    total_nums["all_genome"] = 0
    unknown_nums = {}
    unknown_nums["all_genome"] = 0
    pre_strain = ""
    f_h = open(psortb_file, "r")
    for row in csv.reader(f_h, delimiter="\t"):
        if not row[0].startswith("#"):
            if pre_strain != row[0]:
                subs[row[0]] = {}
                pre_strain = row[0]
                total_nums[row[0]] = 0
                unknown_nums[row[0]] = 0
            if row[5] not in subs[row[0]].keys():
                if row[5] == "Unknown":
                    unknown_nums[row[0]] += 1
                subs[row[0]][row[5]] = 1
                total_nums[row[0]] += 1
            else:
                if row[5] == "Unknown":
                    unknown_nums[row[0]] += 1
                subs[row[0]][row[5]] += 1
                total_nums[row[0]] += 1
            if row[5] not in subs["all_genome"].keys():
                if row[5] == "Unknown":
                    unknown_nums["all_genome"] += 1
                subs["all_genome"][row[5]] = 1
                total_nums["all_genome"] += 1
            else:
                if row[5] == "Unknown":
                    unknown_nums["all_genome"] += 1
                subs["all_genome"][row[5]] += 1
                total_nums["all_genome"] += 1
    f_h.close()
    return subs, total_nums, unknown_nums


def print_file_and_plot(sub, total_nums, unknown_nums,
                        strain, out_stat, prefix_name):
    plot(sub, total_nums[strain], unknown_nums[strain], strain, prefix_name)
    out_stat.write(strain + ":\n")
    out_stat.write("Total including Unknown is {0}; "
                   "Total excluding Unknown is {1}\n".format(
                       total_nums[strain],
                       total_nums[strain] - unknown_nums[strain]))
    for local, num in sub.items():
        if local != "Unknown":
            out_stat.write(
                "\t{0}\t{1}(including Unknown {2}; "
                "excluding Unknonwn {3})\n".format(
                    local, num, float(num) / float(total_nums[strain]),
                    float(num) / (float(total_nums[strain]) - float(
                        unknown_nums[strain]))))
        else:
            out_stat.write("\t{0}\t{1}(including Unknown {2})\n".format(
                local, num, float(num) / float(total_nums[strain])))


def stat_sublocal(psortb_file, prefix_name, stat_file):
    subs, total_nums, unknown_nums = read_table(psortb_file)
    out_stat = open(stat_file, "w")
    if len(subs) > 2:
        print_file_and_plot(subs["all_genome"], total_nums, unknown_nums,
                            "all_genome", out_stat, prefix_name)
    for strain, sub in subs.items():
        if strain != "all_genome":
            print_file_and_plot(sub, total_nums, unknown_nums, strain,
                                out_stat, prefix_name)



import os
import sys
import shutil
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from annogesiclib.sORF_intergenic import get_intergenic
from annogesiclib.sORF_detection import sorf_detection
from annogesiclib.stat_sorf import stat


class sORFDetection(object):
    '''detection of sORF'''

    def __init__(self, args_sorf):
        self.multiparser = Multiparser()
        self.helper = Helper()
        if args_sorf.tsss is not None:
            self.tss_path = os.path.join(args_sorf.tsss, "tmp")
        else:
            self.tss_path = None
        if args_sorf.srnas is not None:
            self.srna_path = os.path.join(args_sorf.srnas, "tmp")
        else:
            self.srna_path = None
        self.gff_output = os.path.join(args_sorf.out_folder, "gffs")
        self.table_output = os.path.join(args_sorf.out_folder, "tables")
        self.tran_path = os.path.join(args_sorf.trans, "tmp")
        self.fasta_path = os.path.join(args_sorf.fastas, "tmp")
        self.all_cand = "all_candidates"
        self.best = "best_candidates"

    def _check_gff(self, gffs):
        for gff in os.listdir(gffs):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(gffs, gff))

    def _check_necessary_files(self, args_sorf):
        if (args_sorf.gffs is None) or \
           (args_sorf.trans is None) or \
           ((args_sorf.tex_wigs is None) and (args_sorf.frag_wigs is None)):
            print("Error: lack required files!!!!")
            sys.exit()
        if args_sorf.utr_detect:
            if (args_sorf.tsss is None):
                print("Error: lack required files for UTR derived"
                      " sORF detection!!!!")
                sys.exit()
        self._check_gff(args_sorf.gffs)
        self.multiparser.parser_gff(args_sorf.gffs, None)
        if args_sorf.tsss is not None:
            self._check_gff(args_sorf.tsss)
            self.multiparser.parser_gff(args_sorf.tsss, "TSS")
            self.multiparser.combine_gff(args_sorf.gffs, self.tss_path,
                                         None, "TSS")
        self._check_gff(args_sorf.trans)
        if args_sorf.srnas is not None:
            self._check_gff(args_sorf.srnas)
            self.multiparser.parser_gff(args_sorf.srnas, "sRNA")
            self.multiparser.combine_gff(args_sorf.gffs, self.srna_path,
                                         None, "sRNA")

    def _start_stop_codon(self, prefixs, args_sorf):
        '''detect the sORF based on start and stop codon 
        and ribosome binding site'''
        for prefix in prefixs:
            if self.srna_path is not None:
                srna_file = os.path.join(self.srna_path,
                                         "_".join([prefix, "sRNA.gff"]))
            else:
                srna_file = None
            if self.tss_path is not None:
                tss_file = os.path.join(self.tss_path,
                                        "_".join([prefix, "TSS.gff"]))
            else:
                tss_file = None
            sorf_detection(os.path.join(self.fasta_path, prefix + ".fa"),
                           srna_file, os.path.join(args_sorf.out_folder,
                           "_".join([prefix, "inter.gff"])), tss_file,
                           os.path.join(args_sorf.wig_path,
                           "_".join([prefix, "forward.wig"])),
                           os.path.join(args_sorf.wig_path,
                           "_".join([prefix, "reverse.wig"])),
                           os.path.join(self.gff_output, self.all_cand,
                           "_".join([prefix, "sORF"])), args_sorf)
            if "_".join([prefix, "sORF_all.gff"]) in os.listdir(
                         os.path.join(self.gff_output, self.all_cand)):
                shutil.move(os.path.join(self.gff_output, self.all_cand,
                            "_".join([prefix, "sORF_all.gff"])),
                            os.path.join(self.gff_output, self.all_cand,
                            "_".join([prefix, "sORF.gff"])))
                shutil.move(os.path.join(self.gff_output, self.all_cand,
                            "_".join([prefix, "sORF_best.gff"])),
                            os.path.join(self.gff_output, self.best,
                            "_".join([prefix, "sORF.gff"])))
                shutil.move(os.path.join(self.gff_output, self.all_cand,
                            "_".join([prefix, "sORF_all.csv"])),
                            os.path.join(self.table_output, self.all_cand,
                            "_".join([prefix, "sORF.csv"])))
                shutil.move(os.path.join(self.gff_output, self.all_cand,
                            "_".join([prefix, "sORF_best.csv"])),
                            os.path.join(self.table_output, self.best,
                            "_".join([prefix, "sORF.csv"])))

    def _remove_tmp(self, args_sorf):
        self.helper.remove_all_content(args_sorf.out_folder, ".gff", "file")
        self.helper.remove_tmp_dir(args_sorf.fastas)
        self.helper.remove_tmp_dir(args_sorf.gffs)
        self.helper.remove_tmp_dir(args_sorf.tsss)
        self.helper.remove_tmp_dir(args_sorf.trans)
        self.helper.remove_tmp_dir(args_sorf.srnas)
        if "temp_wig" in os.listdir(args_sorf.out_folder):
            shutil.rmtree(os.path.join(args_sorf.out_folder, "temp_wig"))
        if "merge_wigs" in os.listdir(args_sorf.out_folder):
            shutil.rmtree(os.path.join(args_sorf.out_folder, "merge_wigs"))

    def _compare_tran_cds(self, args_sorf):
        '''compare transcript and CDS to find the intergenic region'''
        prefixs = []
        for gff in os.listdir(args_sorf.gffs):
            if gff.endswith(".gff"):
                prefix = gff.replace(".gff", "")
                prefixs.append(prefix)
                print("Comparing transcripts and CDSs of {0}".format(prefix))
                get_intergenic(os.path.join(args_sorf.gffs, gff),
                               os.path.join(self.tran_path,
                               "_".join([prefix, "transcript.gff"])),
                               os.path.join(args_sorf.out_folder,
                               "_".join([prefix, "inter.gff"])),
                               args_sorf.utr_detect, args_sorf.hypo)
        return prefixs

    def run_sorf_detection(self, args_sorf):
        if args_sorf.fuzzy_rbs > 6:
            print("Error: --fuzzy_rbs should be equal or less than 6!!")
            sys.exit()
        self._check_necessary_files(args_sorf)
        self.multiparser.parser_gff(args_sorf.trans, "transcript")
        self.multiparser.combine_gff(args_sorf.gffs, self.tran_path,
                                     None, "transcript")
        self.multiparser.parser_fasta(args_sorf.fastas)
        self.multiparser.combine_fasta(args_sorf.gffs, self.fasta_path, None)
        prefixs = self._compare_tran_cds(args_sorf)
        self._start_stop_codon(prefixs, args_sorf)
        for sorf in os.listdir(os.path.join(self.gff_output, self.all_cand)):
            print("Running statistics of {0}".format(sorf))
            if sorf.endswith("_sORF.gff"):
                stat(os.path.join(self.gff_output, self.all_cand, sorf),
                     os.path.join(self.gff_output, self.best, sorf),
                     os.path.join(args_sorf.out_folder, "statistics",
                     "_".join(["stat", sorf.replace(".gff", ".csv")])),
                     args_sorf.utr_detect)
        self._remove_tmp(args_sorf)



from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper


def print_file(datas, out, feature):
    for data in datas:
        if feature not in data.attributes.keys():
            data.attributes[feature] = "NA"
        else:
            data.attributes[feature] = ",".join(data.attributes[feature])
        data.attribute_string = ";".join(
            ["=".join(items) for items in data.attributes.items()])
        out.write("\t".join([data.info_without_attributes,
                  data.attribute_string]) + "\n")


def del_attributes(feature, entry):
    '''Remove to the useless attributes'''
    attributes = {}
    for key, value in entry.attributes.items():
        if feature not in key:
            attributes[key] = value
    return attributes


def srna_sorf_comparison(sRNA_file, sORF_file, sRNA_out, sORF_out):
    '''Comparison of sRNA and sORF. It can be a filter of sRNA detection'''
    sorfs = []
    srnas = []
    out_r = open(sRNA_out, "w")
    out_o = open(sORF_out, "w")
    out_r.write("##gff-version 3\n")
    out_o.write("##gff-version 3\n")
    for entry in Gff3Parser().entries(open(sRNA_file)):
        entry.attributes = del_attributes("sORF", entry)
        srnas.append(entry)
    srnas = sorted(srnas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    for entry in Gff3Parser().entries(open(sORF_file)):
        entry.attributes = del_attributes("sRNA", entry)
        sorfs.append(entry)
    sorfs = sorted(sorfs, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    for srna in srnas:
        for sorf in sorfs:
            if (srna.seq_id == sorf.seq_id) and (srna.strand == sorf.strand):
                if ((srna.start <= sorf.start) and (
                        srna.end >= sorf.end)) or (
                        (srna.start >= sorf.start) and (
                         srna.end <= sorf.end)) or (
                        (srna.start <= sorf.start) and (
                         srna.end >= sorf.start) and (
                         srna.end <= sorf.end)) or (
                        (srna.start >= sorf.start) and (
                         srna.start <= sorf.end) and (
                         srna.end >= sorf.end)):
                    if "sORF" not in srna.attributes.keys():
                        srna.attributes["sORF"] = []
                        strand = Helper().get_strand_name(sorf.strand)
                    srna.attributes["sORF"].append("".join([
                                               "sORF:",
                                               str(sorf.start), "-",
                                               str(sorf.end),
                                               "_", strand]))
                    if "sRNA" not in sorf.attributes.keys():
                        sorf.attributes["sRNA"] = []
                        strand = Helper().get_strand_name(srna.strand)
                    sorf.attributes["sRNA"].append("".join([
                                               "sRNA:",
                                               str(srna.start), "-",
                                               str(srna.end),
                                               "_", strand]))
    print_file(sorfs, out_o, "sRNA")
    print_file(srnas, out_r, "sORF")
    out_r.close()
    out_o.close()



import os
import shutil
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.lib_reader import read_wig, read_libs


def check_start_and_end(start, end, covers):
    if (start - 2) < 0:
        c_start = 0
    else:
        c_start = start - 2
    if (end + 2) > len(covers):
        c_end = len(covers)
    else:
        c_end = end + 2
    return c_start, c_end


def detect_coverage(wigs, tran, infos):
    for strain, conds in wigs.items():
        if strain == tran.seq_id:
            for cond, tracks in conds.items():
                for lib_name, covers in tracks.items():
                    track = lib_name.split("|")[-3]
                    lib_strand = lib_name.split("|")[-2]
                    lib_type = lib_name.split("|")[-1]
                    infos[track] = {"avg": -1, "high": -1, "low": -1}
                    total = 0
                    pos = 0
                    c_start, c_end = check_start_and_end(tran.start, tran.end, covers)
                    for cover in covers[c_start: c_end]:
                        cover_pos = pos + c_start
                        if (cover_pos >= tran.start) and (
                                cover_pos <= tran.end):
                            total = cover + total
                            if cover > infos[track]["high"]:
                                infos[track]["high"] = cover
                            if (cover < infos[track]["low"]) or (
                                    infos[track]["low"] == -1):
                                infos[track]["low"] = cover
                        pos += 1
                    infos[track]["avg"] = (float(total) /
                                           float(tran.end - tran.start + 1))


def print_associate(associate, tran, out):
    if associate in tran.attributes.keys():
        out.write("\t" + tran.attributes[associate])
    else:
        out.write("\tNA")


def print_coverage(trans, out, out_gff, wigs_f, wigs_r, table_best):
    for tran in trans:
        infos = {}
        tran.attributes["detect_lib"] = tran.attributes["detect_lib"].replace(
                                                        "tex_notex", "TEX+/-")
        out.write("\t".join([tran.seq_id, tran.attributes["Name"],
                             str(tran.start), str(tran.end), tran.strand,
                             tran.attributes["detect_lib"]]))
        print_associate("associated_gene", tran, out)
        print_associate("associated_tss", tran, out)
        print_associate("associated_term", tran, out)
        if tran.strand == "+":
            detect_coverage(wigs_f, tran, infos)
        else:
            detect_coverage(wigs_r, tran, infos)
        out.write("\t")
        best = -1
        best_track = ""
        best_cover = {}
        for track, cover in infos.items():
            if not table_best:
                if best != -1:
                    out.write(";")
                out.write("{0}(avg={1})".format(
                          track, str(cover["avg"])))
            if cover["avg"] > best:
                best = cover["avg"]
                best_track = track
                best_cover = cover
        if table_best:
            out.write("{0}(avg={1})".format(
                      best_track, str(best_cover["avg"])))
        out.write("\n")
        new_attrs = {}
        for key, value in tran.attributes.items():
            if ("high_coverage" not in key) and (
                    "low_coverage" not in key):
                new_attrs[key] = value
        new_attrs["best_avg_coverage"] = str(best_cover["avg"])
        attribute_string = ";".join(
            ["=".join(items) for items in new_attrs.items()])
        out_gff.write("\t".join([tran.info_without_attributes,
                                 attribute_string]) + "\n")


def gen_table_transcript(gff_folder, args_tran):
    '''generate the detail table of transcript'''
    libs, texs = read_libs(args_tran.libs, args_tran.merge_wigs)
    for gff in os.listdir(gff_folder):
        if os.path.isfile(os.path.join(gff_folder, gff)):
            wigs_f = read_wig(os.path.join(args_tran.wig_path, "_".join([
                              gff.replace("_transcript.gff", ""),
                              "forward.wig"])), "+", libs)
            wigs_r = read_wig(os.path.join(args_tran.wig_path, "_".join([
                              gff.replace("_transcript.gff", ""),
                              "reverse.wig"])), "-", libs)
            th = open(os.path.join(gff_folder, gff), "r")
            trans = []
            out = open(os.path.join(args_tran.out_folder, "tables",
                       gff.replace(".gff", ".csv")), "w")
            out_gff = open(os.path.join(args_tran.out_folder, "tmp_gff"), "w")
            out_gff.write("##gff-version 3\n")
            out.write("\t".join(["Genome", "Name", "Start", "End", "Strand",
                                 "Detect_lib_type", "Associated_gene",
                                 "Associated_tss", "Associated_term",
                                 "Coverage_details"]) + "\n")
            gff_parser = Gff3Parser()
            for entry in gff_parser.entries(th):
                trans.append(entry)
            print_coverage(trans, out, out_gff, wigs_f, wigs_r,
                           args_tran.table_best)
            out.close()
            out_gff.close()
            shutil.move(os.path.join(args_tran.out_folder, "tmp_gff"),
                        os.path.join(gff_folder, gff))
    if os.path.exists(os.path.join(args_tran.out_folder, "merge_wigs")):
        shutil.rmtree(os.path.join(args_tran.out_folder, "merge_wigs"))



def change_format(input_file, output_file):
    '''change the format of sRNA database'''
    num = 1
    out = open(output_file, "w")
    with open(input_file) as f_h:
        for line in f_h:
            line = line.strip()
            if line.startswith(">"):
                datas = line.split("|")
                if datas[0][1:] == "NA":
                    datas[0] = ">srn_" + str(num)
                    num += 1
                out.write("|".join(datas[:3]) + "\n")
            else:
                out.write(line + "\n")
    out.close()
